# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	include/oes-ldap/dialogs.ycp
# Package:	Configuration of oes-ldap
# Summary:	Dialogs definitions
# Authors:	Novell YaST2 Development Team <support@novell.com>
#
# $Id: dialogs.ycp,v 1.1.2.10 2005/08/02 23:30:06 mclark Exp $
module Yast
  module OesLdapDialogsInclude
    def initialize_oes_ldap_dialogs(include_target)
      Yast.import "UI"

      textdomain "oes-ldap"

      Yast.import "Label"
      Yast.import "Wizard"
      Yast.import "Report"
      Yast.import "Popup"
      Yast.import "NovellUtils"
      Yast.import "EdirUtils"
      Yast.import "NovellValidators"
      Yast.import "NovellAddonInstall"
      #import "Edirectory";
      Yast.import "NovellLDAP"
      Yast.import "Address"
      Yast.include include_target, "oes-ldap/helps.rb"
      Yast.include include_target, "oes-ldap/complex.rb"

      @lastSavedSettings = {}
    end

    #******************* UTILITY ROUTINES for NovellLDAP ***********************
    # Build the table items for the LDAP servers dialog.
    #
    # @param [Array<Hash>] server_list the current list of servers
    # @return table items
    def BuildTableItems(server_list)
      server_list = deep_copy(server_list)
      table_items = []
      id_num = 0
      Builtins.foreach(server_list) do |server_info|
        if Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "") != nil &&
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "") != ""
          type = _("local")
          if NovellLDAP.IsRemoteAddress(
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
              server_list
            )
            type = _("remote")
          end
          table_item = Item(
            Id(id_num),
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
            Ops.get_integer(server_info, "CONFIG_LDAP_PORT", 389),
            Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
            type
          )
          table_items = Builtins.add(table_items, table_item)
          Builtins.y2milestone("adding table item = %1", table_item)
          id_num = Ops.add(id_num, 1)
        end
      end 

      deep_copy(table_items)
    end


    # Build the server list from the UI table.
    #
    # @param [Array<Yast::Term>] table_items the current list of servers in the UI
    # @return server list
    def BuildServerList(table_items)
      table_items = deep_copy(table_items)
      server_list = []
      Builtins.foreach(table_items) do |table_item|
        if Ops.get_string(table_item, 1, "") != nil &&
            Ops.get_string(table_item, 1, "") != ""
          server_info = {
            "CONFIG_LDAP_SERVER_IP"   => Ops.get_string(table_item, 1, ""),
            "CONFIG_LDAP_PORT"        => Builtins.tointeger(
              Ops.get_string(table_item, 2, "389")
            ),
            "CONFIG_LDAP_SECURE_PORT" => Builtins.tointeger(
              Ops.get_string(table_item, 3, "636")
            )
          }
          server_list = Builtins.add(server_list, server_info)
          Builtins.y2milestone("adding server info = %1", server_info)
        end
      end 

      deep_copy(server_list)
    end


    #************************ Common Dialog Widgets *******************
    # ServerListWidget
    #
    # Create a widget for selecting one (or more) LDAP servers from the
    # currently configured list
    #
    # @param server_list The list of servers from which to construct the
    # widget (use nil to check the currently stored list)
    # @param label text to use for the widget
    # @param default_server address specifying the default selected server
    # @param include_local set to true to include local addresses in the list
    # @param type `multi for MultiSelectionBox, `select for SelectionBox or
    # `combo for `ComboBox
    # @return map containg term ldap server list widget and help
    def ServerListWidget(server_list, label, default_server, include_local, type)
      server_list = deep_copy(server_list)
      ret = {}
      widget = nil
      helps = ""
      default_found = false
      sl = deep_copy(server_list)
      if sl == nil
        sl = Ops.get_list(NovellLDAP.Settings, "CONFIG_LDAP_SERVERS_LIST", [])
      end

      Builtins.y2milestone("Original list of LDAP servers: %1", sl)

      # filter the list of servers to be considered if local addresses
      # should not be included
      sl = NovellLDAP.GetRemoteServers(sl) if !include_local
      Builtins.y2milestone("LDAP servers to consider: %1", sl)

      term_list = []
      Builtins.foreach(sl) do |server_info|
        # build the item string
        address = Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
        Builtins.y2milestone("comparing %1 to %2", address, default_server)
        default_found = true if address == default_server
        item = Item(Id(address), address, address == default_server)
        Builtins.y2milestone("Adding %1 to term_list: ", item)
        term_list = Builtins.add(term_list, item)
      end 

      if !default_found && Ops.greater_than(Builtins.size(term_list), 0)
        first = Ops.get(term_list, 0)
        if first != nil
          default_term = Item(
            Ops.get(first, 0),
            Ops.get_string(first, 1, ""),
            true
          )
          Ops.set(term_list, 0, default_term)
        end
      end
      Builtins.y2milestone("LDAP server widget term list: %1", term_list)
      Builtins.y2milestone("LDAP server widget type: %1", type)

      if type == :multi # build the multiselect widget
        widget = MinWidth(
          25,
          MultiSelectionBox(Id(:server_select), label, term_list)
        )
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetMulti", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      elsif type == :select # build the single selection widget
        widget = MinWidth(
          25,
          SelectionBox(Id(:server_select), label, term_list)
        )
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetSingle", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      elsif type == :combo # build the combo box widget
        widget = MinWidth(25, ComboBox(Id(:server_select), label, term_list))
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetCombo", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      end

      Ops.set(ret, "term", widget)
      Ops.set(ret, "helps", helps)
      Ops.set(ret, "size", Builtins.size(term_list))

      deep_copy(ret)
    end
    #************************ Common Dialog Widgets *******************
    # ServerListWidgetAlternateLdapServers
    #
    # Create a widget for selected one (or more) LDAP servers from the
    # currently configured list
    #
    # @param server_list The list of servers from which to construct the
    # widget (use nil to check the currently stored list)
    # @param label text to use for the widget
    # @param default_server address specifying the default selected server
    # @param include_local set to true to include local addresses in the list
    # @param type `multi for MultiSelectionBox, `select for SelectionBox or
    # `combo for `ComboBox
    # @param alternate_ldap_servers_list1 The list of one (or more) servers which were
    # selected already from the available list
    # @return map containg term ldap server list widget and help
    def ServerListWidgetAlternateLdapServers(server_list, label, default_server, include_local, type, alternate_ldap_servers_list1)
      server_list = deep_copy(server_list)
      alternate_ldap_servers_list1 = deep_copy(alternate_ldap_servers_list1)
      ret = {}
      widget = nil
      helps = ""
      default_found = false
      sl = deep_copy(server_list)
      if sl == nil
        sl = Ops.get_list(NovellLDAP.Settings, "CONFIG_LDAP_SERVERS_LIST", [])
      end

      Builtins.y2milestone("Original list of LDAP servers: %1", sl)

      # filter the list of servers to be considered if local addresses
      # should not be included
      sl = NovellLDAP.GetRemoteServers(sl) if !include_local
      Builtins.y2milestone("LDAP servers to consider: %1", sl)

      term_list = []
      Builtins.foreach(sl) do |server_info|
        # build the item string
        address = Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
        Builtins.y2milestone("comparing %1 to %2", address, default_server)
        item = Item(Id(address), address, address == default_server)
        Builtins.y2milestone("Adding %1 to term_list: ", item)
        term_list = Builtins.add(term_list, item)
      end 

      if Ops.greater_than(Builtins.size(term_list), 0)
        alternate_server_list = []
        Builtins.foreach(alternate_ldap_servers_list1) do |alternate_server|
          alternate_server_list = Builtins.add(
            alternate_server_list,
            alternate_server
          )
        end
        idx = 0
        i = 0
        if Ops.greater_than(Builtins.size(alternate_server_list), 0)
          #term item=nil;
          default_term = nil
          Builtins.foreach(term_list) do |item|
            alternate_server = ""
            Builtins.foreach(alternate_server_list) do |alternate_server2|
              default_term = Item(
                Ops.get(item, 0),
                Ops.get_string(item, 1, ""),
                Ops.get_string(item, 1, "") == alternate_server2 ? true : false
              )
              raise Break if Ops.get_string(item, 1, "") == alternate_server2
            end
            Ops.set(term_list, idx, default_term)
            idx = Ops.add(idx, 1)
          end
        end
      end
      Builtins.y2milestone("LDAP server widget term list: %1", term_list)
      Builtins.y2milestone("LDAP server widget type: %1", type)

      if type == :multi # build the multiselect widget
        widget = MinWidth(
          25,
          MultiSelectionBox(Id(:server_select), label, term_list)
        )
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetMulti", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      elsif type == :select # build the single selection widget
        widget = MinWidth(
          25,
          SelectionBox(Id(:server_select), label, term_list)
        )
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetSingle", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      elsif type == :combo # build the combo box widget
        widget = MinWidth(25, ComboBox(Id(:server_select), label, term_list))
        helps = Builtins.sformat(
          Ops.get_string(@HELPS, "ServerListWidgetCombo", ""),
          label
        )
        helps = "" if helps == nil # avoid nil value
      end

      Ops.set(ret, "term", widget)
      Ops.set(ret, "helps", helps)
      Ops.set(ret, "size", Builtins.size(term_list))

      deep_copy(ret)
    end


    #************************ New Multi-Server LDAP Dialogs *******************

    def ValidateCredentials(ip)
      returnCode = 0

      Builtins.y2milestone(
        "oesldap.ValidateCredentials:entering dialog with IP / Hostname: %1",
        ip
      )
      admin_password = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      # determine IP addresses
      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      primary_server = NovellLDAP.GetAddressServer(ip, server_list)

      retCode = NovellValidators.ValidateCredentials(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", ""),
        admin_password,
        Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
        Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636),
        true
      )
      if retCode != 0
        returnCode = retCode

        if retCode == -666
          err = _(
            "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
              "This may indicate a problem with LDAP on the target server.\n" +
              "If you are confident that your credentials are correct you may continue,\n" +
              "otherwise please determine if LDAP is running correctly on the target system\n" +
              "Do you want to continue?"
          )
          yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err)
          returnCode = 3 if !yes
        else
          err = Builtins.sformat(
            _(
              "User credentials failed to validate Using:\n" +
                "      Server: %1\n" +
                "      User: %2\n" +
                "      LDAP port: %3\n" +
                "Do you want to continue anyway?"
            ),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", ""),
            Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
          )
          yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
          returnCode = 3 if !yes
        end
      end
      Builtins.y2milestone(
        "oesldap.ValidateCredentials:returning %1",
        returnCode
      )
      returnCode
    end


    # AdminPasswordDialog2
    #
    # Ask for the admin password
    def AdminPasswordDialog2(doCredentialCheck)
      Builtins.y2milestone("oesldap.AdminPasswordDialog:entering dialog.")
      admin_password = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      # build the UI
      contents = HVSquash(
        VBox(
          Left(
            Label(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      _("eDirectory Tree: "),
                      Ops.get_string(
                        NovellLDAP.Settings,
                        "CONFIG_LDAP_TREE_NAME",
                        ""
                      )
                    ),
                    "\n"
                  ),
                  _("Admin User: ")
                ),
                Ops.get_string(
                  NovellLDAP.Settings,
                  "CONFIG_LDAP_ADMIN_CONTEXT",
                  ""
                )
              )
            )
          ),
          Left(
            Password(
              Id(:admin_password),
              _("&Please enter the admin password:"),
              admin_password
            )
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:admin_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          NovellUtils.channelUpgrade_abort = true
          break
        elsif ret == :ok
          # get the user input
          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )

          # validate the password
          if admin_password == nil || admin_password == ""
            Report.Error(_("The admin password cannot be blank."))
            next
          else
            server_info = Ops.get(NovellLDAP.GetRemoteServers(nil), 0, {})
            if server_info == nil || server_info == {}
              server_info = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
              if server_info != nil && server_info != {}
                edir_configured = Convert.to_string(
                  SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
                )
                edir_configured = Builtins.tolower(edir_configured)
                server_info = {} if edir_configured == "no"
              end
            end
            if server_info != nil && server_info != {} &&
                doCredentialCheck == true
              retCode = NovellValidators.ValidateCredentials(
                Ops.get_string(
                  NovellLDAP.Settings,
                  "CONFIG_LDAP_ADMIN_CONTEXT",
                  ""
                ),
                admin_password,
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
                true
              )
              if retCode != 0
                if retCode == -666
                  err = _(
                    "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
                      "This may indicate a problem with LDAP on the target server.\n" +
                      "If you are confident that your credentials are correct you may continue,\n" +
                      "otherwise please determine if LDAP is running correctly on the target system\n" +
                      "Do you want to continue?"
                  )
                  yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err)
                  next if !yes
                else
                  err = Builtins.sformat(
                    _(
                      "User credentials failed to validate Using:\n" +
                        "      Server: %1\n" +
                        "      User: %2\n" +
                        "      LDAP port: %3\n" +
                        "Do you want to continue anyway?"
                    ),
                    Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                    Ops.get_string(
                      NovellLDAP.Settings,
                      "CONFIG_LDAP_ADMIN_CONTEXT",
                      ""
                    ),
                    Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
                  )
                  yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
                  next if !yes
                end
              end
            end
          end

          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            admin_password
          )
          break
        end
      end
      UI.CloseDialog

      nil
    end

    # AdminPasswordDialog2channelUpgrade
    #
    # Ask for the admin password
    def AdminPasswordDialog2ChannelUpgrade(doCredentialCheck)
      Builtins.y2milestone("oesldap.AdminPasswordDialog:entering dialog.")
      admin_password = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      # build the UI
      contents = HVSquash(
        VBox(
          Left(
            Label(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      _("eDirectory Tree: "),
                      Ops.get_string(
                        NovellLDAP.Settings,
                        "CONFIG_LDAP_TREE_NAME",
                        ""
                      )
                    ),
                    "\n"
                  ),
                  _("Admin User: ")
                ),
                Ops.get_string(
                  NovellLDAP.Settings,
                  "CONFIG_LDAP_ADMIN_CONTEXT",
                  ""
                )
              )
            )
          ),
          Left(
            Password(
              Id(:admin_password),
              _("&Please enter the admin password:"),
              admin_password
            )
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:admin_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          NovellUtils.channelUpgrade_abort = true
          break
        elsif ret == :ok
          # get the user input
          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )

          # validate the password
          if admin_password == nil || admin_password == ""
            Report.Error(_("The admin password cannot be blank."))
            next
          else
            server_info = Ops.get(NovellLDAP.GetRemoteServers(nil), 0, {})
            if server_info == nil || server_info == {}
              server_info = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
              if server_info == nil && server_info == {}
                edir_configured = Convert.to_string(
                  SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
                )
                edir_configured = Builtins.tolower(edir_configured)
                server_info = {} if edir_configured == "no"
              end
            end
            if server_info != nil && server_info != {} &&
                doCredentialCheck == true
              retCode = NovellValidators.ValidateCredentials(
                Ops.get_string(
                  NovellLDAP.Settings,
                  "CONFIG_LDAP_ADMIN_CONTEXT",
                  ""
                ),
                admin_password,
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
                true
              )
              if NovellUtils.channelUpgrade
                if retCode != 0
                  Report.Error(NovellUtils.channelUpgrade_error)
                  UI.ChangeWidget(Id(:admin_password), :Value, "")
                  UI.SetFocus(Id(:admin_password))
                  next
                end
              end
              retCode1 = NovellUtils.NDSPasswordCheck(admin_password)
              if retCode1 != 0
                UI.ChangeWidget(Id(:admin_password), :Value, "")
                UI.SetFocus(Id(:admin_password))
                next
              end
            end
          end

          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            admin_password
          )
          break
        end
      end
      UI.CloseDialog
      #nil
      return true
    end


    # AdminPasswordDialog2
    #
    # Ask for the admin password
    def AdminPasswordDialog
      AdminPasswordDialog2(true)

      nil
    end
    # AdminPasswordDialog2
    #
    # Ask for the admin password
    def AdminPasswordDialogChannelUpgrade
      return AdminPasswordDialog2ChannelUpgrade(true)

      nil
    end

    # CommonProxyPasswordDialog
    #
    # Ask for the common proxy password when the user entered password is not compliant with password policy
    def CommonProxyPasswordDialog
      Builtins.y2milestone("oesldap.CommonProxyPasswordDialog:entering dialog.")
      proxy_password = nil

      # build the UI
      contents = HVSquash(
        VBox(
          Password(
            Id(:proxy_password),
            _(
              "The password entered for the proxy user is not compliant with password policy.\n&Please enter a valid common proxy password:"
            ),
            ""
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:proxy_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          break
        elsif ret == :ok
          # get the user input
          proxy_password = Convert.to_string(
            UI.QueryWidget(Id(:proxy_password), :Value)
          )

          # validate the password
          if proxy_password == nil || proxy_password == ""
            Report.Error(_("The common proxy password cannot be blank."))
            next
          end
          if !NovellValidators.ValidatePassword(proxy_password)
            UI.ChangeWidget(Id(:proxy_password), :Value, "")
            UI.SetFocus(Id(:proxy_password))
            next
          end
          break
        end
      end
      UI.CloseDialog

      proxy_password
    end

    # UpgradeReplicaAddressdDialog
    #
    # Ask to verify or re-enter the replica address during an eDir upgrade.
    def UpgradeReplicaAddressdDialog(replica_address)
      Builtins.y2milestone(
        "oesldap.UpgradeReplicaAddressdDialog:entering dialog."
      )

      # build the UI
      contents = HVSquash(
        VBox(
          TextEntry(
            Id(:replica_address),
            _(
              "Please verify that the IP address / hostname of a server with a replica is correct:"
            ),
            replica_address
          ),
          HBox(Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)))
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:replica_address))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :ok
          replica_address = Convert.to_string(
            UI.QueryWidget(Id(:replica_address), :Value)
          )

          if !Address.Check4(replica_address)
            if replica_address != ""
              Popup.Error(Address.Valid4)
            else
              Popup.Error(
                _(
                  "Enter the IP address / hostname of an existing server with a replica in the tree."
                )
              )
            end
            next
          else
            local_addresses = Convert.convert(
              NovellUtils.GetAllStaticIPAddrs,
              :from => "list",
              :to   => "list <string>"
            )
            if Builtins.contains(local_addresses, replica_address)
              Report.Error(
                _(
                  "The replica server address must be a remote (non-local) address."
                )
              )
              replica_address = ""
              UI.ChangeWidget(Id(:replica_address), :Value, replica_address)
              UI.SetFocus(Id(:replica_address))
              next
            end
          end
          if !NovellUtils.Ping_IP(replica_address)
            erMsg = Builtins.sformat(
              _(
                "The specified IP address / hostname (%1) is either not reachable or unresolvable.\nDo you want to continue anyway?"
              ),
              replica_address
            )

            next if Popup.ContinueCancel(erMsg) == false
          end
          Builtins.y2milestone(
            "NovellLDAPUI:UpgradeReplicaAddressDialog:Value from upgrade dialog is %1.",
            replica_address
          )

          break
        end
      end
      UI.CloseDialog

      replica_address
    end

    # AdminPasswordDialog2
    #
    # Ask for the Domain Admin password
    def XadDomainAdminPasswordDialog
      Builtins.y2milestone(
        "oesldap.XadDomainAdminPasswordDialog:entering dialog."
      )

      # build the UI
      contents = HVSquash(
        VBox(
          Password(
            Id(:admin_password),
            _("&Please enter the DSFW Domain Admin password for this server:"),
            ""
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:admin_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          NovellUtils.channelUpgrade_abort = true
          break
        elsif ret == :ok
          # get the user input
          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )

          # validate the password
          if admin_password == nil || admin_password == ""
            Report.Error(_("The DSFW Domain Admin password cannot be blank."))
            next
          end
          if !NovellValidators.ValidatePassword(admin_password)
            UI.ChangeWidget(Id(:admin_password), :Value, "")
            UI.SetFocus(Id(:admin_password))
            next
          end
          break
        end
      end
      UI.CloseDialog

      admin_password
    end

    # XadTreeAdminPasswordDialog
    #
    # Ask for the admin password
    def XadTreeAdminPasswordDialog
      Builtins.y2milestone(
        "oesldap.XadTreeAdminPasswordDialog:entering dialog."
      )
      admin_password = Ops.get_string(
        NovellLDAP.Settings,
        "XAD_TREE_ADMIN_PASSWORD",
        ""
      )

      # build the UI
      contents = HVSquash(
        VBox(
          Left(
            Label(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      _("eDirectory Tree: "),
                      Ops.get_string(
                        NovellLDAP.Settings,
                        "CONFIG_LDAP_TREE_NAME",
                        ""
                      )
                    ),
                    "\n"
                  ),
                  _("Admin User: ")
                ),
                Ops.get_string(
                  NovellLDAP.Settings,
                  "XAD_TREE_ADMIN_CONTEXT",
                  ""
                )
              )
            )
          ),
          Password(
            Id(:admin_password),
            _("&Please enter the tree admin password:"),
            admin_password
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:admin_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          NovellUtils.channelUpgrade_abort = true
          break
        elsif ret == :ok
          # get the user input
          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )

          # validate the password
          if admin_password == nil || admin_password == ""
            Report.Error(_("The admin password cannot be blank."))
            next
          else
            server_info = Ops.get(NovellLDAP.GetRemoteServers(nil), 0, {})
            if server_info == nil || server_info == {}
              server_info = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
              if server_info != nil && server_info != {}
                edir_configured = Convert.to_string(
                  SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
                )
                edir_configured = Builtins.tolower(edir_configured)
                server_info = {} if edir_configured == "no"
              end
            end
            if server_info != nil && server_info != {}
              retCode = NovellValidators.ValidateCredentials(
                Ops.get_string(
                  NovellLDAP.Settings,
                  "XAD_TREE_ADMIN_CONTEXT",
                  ""
                ),
                admin_password,
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
                true
              )
              if retCode != 0
                if retCode == -666
                  err = _(
                    "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
                      "This may indicate a problem with LDAP on the target server.\n" +
                      "If you are confident that your credentials are correct you may continue,\n" +
                      "otherwise please determine if LDAP is running correctly on the target system\n" +
                      "Do you want to continue?"
                  )
                  yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err)
                  next if !yes
                else
                  err = Builtins.sformat(
                    _(
                      "User credentials failed to validate Using:\n" +
                        "      Server: %1\n" +
                        "      User: %2\n" +
                        "      LDAP port: %3\n" +
                        "Do you want to continue anyway?"
                    ),
                    Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                    Ops.get_string(
                      NovellLDAP.Settings,
                      "XAD_TREE_ADMIN_CONTEXT",
                      ""
                    ),
                    Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
                  )
                  yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
                  next if !yes
                end
              end
            end
          end

          Ops.set(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            admin_password
          )
          break
        end
      end
      UI.CloseDialog

      nil
    end

    # XadTreeAdminPasswordDialogChannelUpgrade
    #
    # Ask for the admin passwordChnnelUpgrade
    def XadTreeAdminPasswordDialogChannelUpgrade
      Builtins.y2milestone(
        "oesldap.XadTreeAdminPasswordDialog:entering dialog."
      )
      admin_password = Ops.get_string(
        NovellLDAP.Settings,
        "XAD_TREE_ADMIN_PASSWORD",
        ""
      )

      # build the UI
      contents = HVSquash(
        VBox(
          Left(
            Label(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      _("eDirectory Tree: "),
                      Ops.get_string(
                        NovellLDAP.Settings,
                        "CONFIG_LDAP_TREE_NAME",
                        ""
                      )
                    ),
                    "\n"
                  ),
                  _("Admin User: ")
                ),
                Ops.get_string(
                  NovellLDAP.Settings,
                  "XAD_TREE_ADMIN_CONTEXT",
                  ""
                )
              )
            )
          ),
          Password(
            Id(:admin_password),
            _("&Please enter the tree admin password:"),
            admin_password
          ),
          HBox(
            Left(PushButton(Id(:ok), Opt(:default), Label.OKButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:admin_password))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          NovellUtils.channelUpgrade_abort = true
          break
        elsif ret == :ok
          # get the user input
          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )

          # validate the password
          if admin_password == nil || admin_password == ""
            Report.Error(_("The admin password cannot be blank."))
            next
          else
            server_info = Ops.get(NovellLDAP.GetRemoteServers(nil), 0, {})
            if server_info == nil || server_info == {}
              server_info = Ops.get(NovellLDAP.GetLocalServers(nil), 0, {})
              if server_info == nil && server_info == {}
                edir_configured = Convert.to_string(
                  SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
                )
                edir_configured = Builtins.tolower(edir_configured)
                server_info = {} if edir_configured == "no"
              end
            end
            if server_info != nil && server_info != {}
              retCode = NovellValidators.ValidateCredentials(
                Ops.get_string(
                  NovellLDAP.Settings,
                  "XAD_TREE_ADMIN_CONTEXT",
                  ""
                ),
                admin_password,
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
                true
              )
              if NovellUtils.channelUpgrade
                if retCode != 0
                  Report.Error(NovellUtils.channelUpgrade_error)
                  UI.ChangeWidget(Id(:admin_password), :Value, "")
                  UI.SetFocus(Id(:admin_password))
                  next
                end
              end
            end
          end

          Ops.set(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_PASSWORD",
            admin_password
          )
          break
        end
      end
      UI.CloseDialog
      return true
#      nil
    end

    # AddServerDialog
    #
    # Ask for information to add a new LDAP server
    # to the list of available LDAP servers
    #
    # @param server_list the current list of servers
    # @param server_type `local, `remote, or `any
    # @return map dialog result
    def AddServerDialog(server_list, server_type)
      server_list = deep_copy(server_list)
      new_server = {}
      address = ""
      port = 389
      secure_port = 636
      type = server_type
      type = :any if type == nil

      # build the address term
      local_servers = NovellLDAP.GetLocalServers(server_list)
      local_available = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      local_in_list = NovellLDAP.GetAddresses(local_servers)
      Builtins.y2milestone("Bound static addresses: %1", local_available)
      Builtins.y2milestone("Local addresses in list: %1", local_in_list)
      local_available = Builtins.filter(local_available) do |address2|
        !Builtins.contains(local_in_list, address2)
      end
      label = _("S&erver Address")
      address_entry = nil
      if type == :local
        if local_available == []
          Popup.Error(
            _("All local static addresses have already been configured.")
          )
          return deep_copy(new_server)
        end
        address_entry = ComboBox(
          Id(:address),
          Opt(:hstretch),
          Ops.add(_("Local "), label),
          local_available
        )
      elsif type == :remote
        address_entry = TextEntry(
          Id(:address),
          Opt(:hstretch),
          Ops.add(_("Remote "), label),
          address
        )
      elsif type == :any
        if local_available != []
          address_entry = ComboBox(
            Id(:address),
            Opt(:hstretch, :editable),
            label,
            local_available
          )
        else
          address_entry = TextEntry(
            Id(:address),
            Opt(:hstretch),
            label,
            address
          )
        end
      end

      # build the UI
      contents = HVSquash(
        VBox(
          address_entry,
          IntField(Id(:port), _("&LDAP Port"), 1, 65535, port),
          IntField(
            Id(:secure_port),
            _("Sec&ure LDAP Port"),
            1,
            65535,
            secure_port
          ),
          HBox(
            Left(PushButton(Id(:add), Opt(:default), Label.AddButton)),
            Left(PushButton(Id(:cancel), Label.CancelButton))
          )
        )
      )

      # open the dialog
      UI.OpenDialog(Opt(:decorated), contents)
      UI.SetFocus(Id(:address))

      # handle the events
      while true
        ret = UI.UserInput
        if ret == :cancel
          new_server = {}
          break
        elsif ret == :add
          # get the user input
          address = Convert.to_string(UI.QueryWidget(Id(:address), :Value))
          port = Convert.to_integer(UI.QueryWidget(Id(:port), :Value))
          secure_port = Convert.to_integer(
            UI.QueryWidget(Id(:secure_port), :Value)
          )

          # validate the address
          if address == nil || address == ""
            Report.Error(_("The server address cannot be blank."))
            UI.SetFocus(Id(:address))
            next
          elsif !NovellValidators.ValidateDNSorIP(address)
            # the validator has it's own error message
            address = ""
            UI.ChangeWidget(Id(:address), :Value, address)
            UI.SetFocus(Id(:address))
            next
          elsif NovellLDAP.AddressExists(address, server_list)
            Report.Error(_("The server address entered already exists."))
            address = ""
            UI.ChangeWidget(Id(:address), :Value, address)
            UI.SetFocus(Id(:address))
            next
          elsif type == :remote && Builtins.contains(local_available, address)
            Report.Error(_("The server address may NOT be a local address."))
            address = ""
            UI.ChangeWidget(Id(address), :Value, address)
            UI.SetFocus(Id(:address))
            next
          end

          # create the map of values
          new_server = {
            "CONFIG_LDAP_SERVER_IP"   => address,
            "CONFIG_LDAP_PORT"        => port,
            "CONFIG_LDAP_SECURE_PORT" => secure_port
          }
          break
        end
      end
      UI.CloseDialog
      deep_copy(new_server)
    end

    # LDAPServersDialog
    #
    # Ask for information on the available LDAP servers to
    # use for Open Enterprise Server services
    #
    # @return dialog result
    def LDAPServersDialog
      # LDAPDefaultServerDialog caption
      caption = _("Configured LDAP Servers")

      Builtins.y2milestone("Entering LDAPServersDialog()")

      remoteEdir = false
      # LUM or NCS installed without local edir
      if (NovellPatterns.IsPatternInstalled("novell-lum") || NovellPatterns.IsPatternInstalled("novell-ncs")) && !NovellPatterns.IsPatternInstalled("novell-edirectory")
        remoteEdir = true
      end

      # Set fields
      tree = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", "")
      adminName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )
      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )
      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )

      commonProxy_context = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_COMMON_PROXY_CONTEXT",
        ""
      )

      #FIXME: is this needed?
      # if( OesLdap::autoyast_tool == true )
      #     {
      #       if( tree == "" )
      #           tree = OesLdap::lastSaved["CONFIG_EDIR_TREE_NAME"]:"";
      #
      #       if( adminName == "" )
      #           adminName = OesLdap::lastSaved["CONFIG_LDAP_ADMIN_CONTEXT"]:"";
      #
      #       if( adminPassword == "" )
      #           adminPassword = OesLdap::lastSaved["CONFIG_LDAP_ADMIN_PASSWORD"]:"";
      #     }
      # LDAP Servers Dialog
      contents = HVCenter(
        VBox(
          HSpacing(Opt(:hstretch), 45),
          TextEntry(Id(:tree), _("&eDirectory Tree Name"), tree),
          TextEntry(Id(:adminName), Opt(:notify, :immediate), _("Ad&min name and context"), adminName),
          HBox(MinWidth(45,Password(Id(:adminPassword), _("Admin &password"), adminPassword))),
          ReplacePoint(Id("replacepoint_commonproxy_context"), Empty()),
          Label(_("Configured LDAP Servers")),
          HBox(
            MinWidth(
              55,
              Table(
                Id(:gui_table),
                Opt(:notify, :immediate),
                # table header
                Header(
                  _("IP Address / Hostname"),
                  _("LDAP Port"),
                  _("Secure LDAP Port"),
                  _("Server Type")
                ),
                #Table Items
                BuildTableItems(server_list)
              )
            ),
            VSpacing(Opt(:vstretch), 10)
          ),
          # button label
          HBox(
            PushButton(Id(:add_button), Label.AddButton),
            #`PushButton(`id(`editBttn), Label::EditButton()),
            PushButton(Id(:delete_button), Label.DeleteButton)
          ),
          VSpacing(0.4)
        )
      )


      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "ldap_server_config", ""),
        Label.BackButton,
        Label.NextButton
      )

      if tree == ""
        UI.SetFocus(Id(:tree))
      elsif adminName == ""
        UI.SetFocus(Id(:adminName))
      elsif adminPassword == ""
        UI.SetFocus(Id(:adminPassword))
      end

      if remoteEdir
        UI.ReplaceWidget(
          Id("replacepoint_commonproxy_context"),
          TextEntry(Id(:commonProxy_context), _("Common Proxy user context"), commonProxy_context)
        )
      end

      ret = nil
      # Establish writeability of fields
      # 	if (size(tree) > 0)
      # 	{
      # 	    UI::ChangeWidget(`id(`tree), `Enabled, false);
      # 	}
      # 	if (size(adminName) > 0)
      # 	{
      # 	    UI::ChangeWidget(`id(`adminName), `Enabled, false);
      # 	}
      # 	if (size(adminPassword) > 0)
      # 	{
      # 	    UI::ChangeWidget(`id(`adminPassword), `Enabled, false);
      # 	}
      # Begin main dailog processing

      while true
        ret = UI.UserInput
        # Read Back the Data
        tree = Convert.to_string(UI.QueryWidget(Id(:tree), :Value))
        adminName = Convert.to_string(UI.QueryWidget(Id(:adminName), :Value))
        adminPassword = Convert.to_string(
          UI.QueryWidget(Id(:adminPassword), :Value)
        )
        server_list = BuildServerList(
          Convert.convert(
            UI.QueryWidget(Id(:gui_table), :Items),
            :from => "any",
            :to   => "list <term>"
          )
        )

        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          #make sure an LDAP server has been defined.
          if Builtins.size(server_list) == 0
            if !NovellLDAP.NeedLDAP
              Ops.set(
                NovellAddonInstall.products_to_configure,
                "oes-ldap",
                false
              )
            else
              leave_empty = Popup.ContinueCancel(
                _(
                  "No LDAP servers have been specified.  This may prevent other services from properly configuring.  Continue anyway?"
                )
              )
              if !leave_empty
                next
              else
                Ops.set(
                  NovellAddonInstall.products_to_configure,
                  "oes-ldap",
                  false
                )
              end
            end
          else
            next if !NovellValidators.ValidateTreeName(tree)
            next if !NovellValidators.ValidateFDN(adminName, false)
            next if !NovellValidators.ValidatePassword(adminPassword)
          end

          # Save the data
          Ops.set(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", tree)
          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            NovellUtils.Nds2Ldap(adminName)
          )
          Ops.set(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_PASSWORD",
            adminPassword
          )
          Ops.set(NovellLDAP.Settings, "CONFIG_LDAP_SERVERS_LIST", server_list)

          if remoteEdir
            commonProxy_context = Convert.to_string(UI.QueryWidget(Id(:commonProxy_context), :Value))
            commonProxy_context = NovellUtils.Nds2Ldap(commonProxy_context)
            if !NovellValidators.ValidateContext(commonProxy_context)
              name = commonProxy_context
              name = NovellUtils.Strip(name, " ")
              if name == nil || name == "" || name == " "
                Popup.Error(_("A valid LDAP context name must be entered."))
              end
              UI.SetFocus(Id(:commonProxy_context))
              next
            end
            Ops.set(NovellLDAP.Settings,"CONFIG_COMMON_PROXY_CONTEXT", commonProxy_context)
            Ops.set(NovellLDAP.Settings, "CONFIG_LDAP_SERVERS_LIST", server_list)
            cmd = Builtins.sformat("/opt/novell/proxymgmt/bin/cp_retrieve_proxy_cred username")
            exit = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
            if exit != 0
              retValue = ConfigureCommonProxy(server_list)
              Builtins.y2milestone("Common Proxy created status : %1",retValue)
            end
          end
          # FIXME: is this needed?
          # if (OesLdap::autoyast_tool == true)
          # 			{
          # 				OesLdap::SetLDAPEdirSettingsinLastSaved(lastSavedSettings);
          # 			}
          break
        elsif ret == :back
          break
        #if an item is clicked in the table
        elsif ret == :gui_table
          next
        elsif ret == :adminName
          adminName = Convert.to_string(UI.QueryWidget(Id(:adminName), :Value))
          if remoteEdir
            commonProxy_context = NovellUtils.GetContextNDS(adminName)
            UI.ChangeWidget(Id(:commonProxy_context), :Value, commonProxy_context)
          end
          next
        elsif ret == :add_button
          progress_disable = Progress.set(false)
          server_info = AddServerDialog(server_list, :any)
          if server_info != nil && server_info != {}
            val = NovellValidators.ValidateCredentials(
              adminName,
              adminPassword,
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
              Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
              true
            )
            Builtins.y2milestone("ValidateCredentials() returned %1", val)
            if val == 0
              server_list = Builtins.add(server_list, server_info)
              UI.ChangeWidget(
                Id(:gui_table),
                :Items,
                BuildTableItems(server_list)
              )
            else
              # Try a non-secure login
              val2 = NovellValidators.ValidateCredentials(
                adminName,
                adminPassword,
                Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                Ops.get_integer(server_info, "CONFIG_LDAP_PORT", 389),
                true
              )
              if val2 != 0
                err = Builtins.sformat(
                  _(
                    "User credentials failed to validate Using:\n" +
                      "      Server: %1\n" +
                      "      User: %2\n" +
                      "      LDAP ports: %3, %4\n" +
                      "Do you want to continue anyway?"
                  ),
                  Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
                  adminName,
                  Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636),
                  Ops.get_integer(server_info, "CONFIG_LDAP_PORT", 389)
                )
                yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
                if yes
                  server_list = Builtins.add(server_list, server_info)
                  UI.ChangeWidget(
                    Id(:gui_table),
                    :Items,
                    BuildTableItems(server_list)
                  )
                end
              end
            end
          end
          Progress.set(progress_disable)
          next
        elsif ret == :delete_button
          index = Convert.to_integer(
            UI.QueryWidget(Id(:gui_table), :CurrentItem)
          )
          Builtins.y2milestone("Entry to delete: %1", index)
          if index != nil && Ops.greater_or_equal(index, 0) &&
              Ops.less_than(index, Builtins.size(server_list))
            server_list = Builtins.remove(server_list, index)
            UI.ChangeWidget(
              Id(:gui_table),
              :Items,
              BuildTableItems(server_list)
            )
          end
          next
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      Builtins.y2milestone("LDAPServersDialog() returning %1", ret)
      deep_copy(ret)
    end

    def ConfigureCommonProxy(server_list)
      remoteIp = ""
      sPort = ""
      ret = false
      admin = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )
      adminPassword = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )

      Builtins.foreach(server_list) do |server_info|
        if NovellLDAP.IsRemoteAddress(
          Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", ""),
          server_list
        )
          remoteIp = Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
          if remoteIp != ""
          Builtins.y2milestone("Remote LDAP Server IP is %1",remoteIp)
          sPort = Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
          break
          end
        end
      end

      proxyName = ""
      proxyContext = ""

      assignPolicy = "yes"
      serverContext = Ops.get_string(NovellLDAP.Settings,"CONFIG_COMMON_PROXY_CONTEXT","")
      if serverContext == ""
        serverContext = NovellUtils.GetContextNDS(admin)
      end
      hostName = NovellUtils.GetHostname
      proxyName = Ops.add("cn=OESCommonProxy_", hostName)
      proxyContext = serverContext
      proxyPassword = NovellUtils.GenerateRandomPassword # generate a random password between 20 and 50 characters

      Ops.set(NovellLDAP.Settings,"CONFIG_LDAP_ASSIGN_POLICY",assignPolicy)
      Ops.set(NovellLDAP.Settings,"CONFIG_LDAP_PROXY_CONTEXT",Ops.add(Ops.add(proxyName, ","), proxyContext))
      Ops.set(NovellLDAP.Settings,"CONFIG_LDAP_REMOTE_SERVER_IP",remoteIp)
      Ops.set(NovellLDAP.Settings,"CONFIG_LDAP_PROXY_PASSWORD",proxyPassword)
      Ops.set(NovellLDAP.Settings,"CONFIG_LDAP_SECURE_PORT",sPort)

      Yast.import "Edirectory"
      if Edirectory.SSRootCert
        ret = Edirectory.ConfigureProxyUser(admin, adminPassword)
      end
     ret
    end
  end
end
