# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellLDAP.ycp
# Package:	Configuration of oes-ldap
# Summary:	NovellLDAP settings, input and output functions
# Authors:	Michael G. Fritch <mgfritch@novell.com>
#
# $Id: NovellLDAP.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Representation of the configuration of oes-ldap.
# Input and output routines.
require "yast"

module Yast
  class NovellLDAPClass < Module
    def main
      textdomain "oes-ldap"

      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Message"
      Yast.import "NovellAddonInstall"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "Popup"
      # Name of this module to display on the UI
      @product_name = _("LDAP Servers") # FIXME: what should the title be?

      # SCR path to the sysconfig agent for this module
      @sysconfig_agent = path(".sysconfig.novell.ldap_servers")

      # Filesystem path to the config directory for this modules config files
      @config_dir = "/etc/sysconfig/novell/ldap_servers/"

      # Filesystem path to the global config file for this module
      @global_cfgfile = "/etc/sysconfig/novell/oes-ldap"

      # Filename of the server template config file for this module
      @template_cfg_name = "ldap_server.template"

      # Filesystem path to the server template config file for this module
      @template_cfgfile = Builtins.sformat(
        "%1%2",
        @config_dir,
        @template_cfg_name
      )

      # List of reserved config filenames that cannot be used to define an ldap server
      @reserved_cfgfile_names = [@template_cfg_name]

      # Global Settings Map
      @Settings = {}

      # Data was modified?
      @modified = false


      @proposal_valid = false

      @globals_configured = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
      @auto_clone = false
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end


    # Read all the module specific info from the config file
    #
    # @return [Hash{String => Object}] of config file settings
    def Info
      info = NovellAddonInstall.ReadConfigFileSettings(@global_cfgfile)
      Ops.set(info, "PRODUCT_NAME", @product_name)
      Builtins.y2milestone("NovellLDAP::Info() returned: %1", info)
      deep_copy(info)
    end

    def NeedLDAP
      ret = false
      if NovellPatterns.IsPatternInstalled("novell-dhcp") ||
          NovellPatterns.IsPatternInstalled("novell-dns") ||
          NovellPatterns.IsPatternInstalled("novell-domain-services") ||
          #novell-edirectory
          NovellPatterns.IsPatternInstalled("novell-ftp") ||
          #novell-imanager
          NovellPatterns.IsPatternInstalled("novell-iprint") ||
          #novell-kdc
          NovellPatterns.IsPatternInstalled("novell-lum") ||
          NovellPatterns.IsPatternInstalled("novell-ncp") ||
          NovellPatterns.IsPatternInstalled("novell-ncs") ||
          NovellPatterns.IsPatternInstalled("novell-nrm") ||
          NovellPatterns.IsPatternInstalled("novell-nss") ||
          NovellPatterns.IsPatternInstalled("novell-samba") ||
          NovellPatterns.IsPatternInstalled("novell-sms")
        ret = true
      end
      #novell-vix
      #novell-web-services
      #novell-welcome
      #novell-xtier
      #novell-yast2
      ret
    end

    def CheckLDAPBlocker(checkPassword)
      servers = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", [])
      if NeedLDAP() &&
          (Builtins.size(servers) == 0 ||
            Ops.get_string(@Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "") == "" ||
            Ops.get_string(@Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "") == "" && checkPassword ||
            Ops.get_string(@Settings, "CONFIG_LDAP_TREE_NAME", "") == "")
        return true
      else
        return false
      end
    end


    # Read a setting (option) from a specified config file
    # Removes any extra quotes around values and avoids passing a nil value
    #
    # @param [Yast::Path] configpath scr path to the config file
    # @param [String] optname string key of the setting(option) we are reading
    # @return [String] value of the specfied setting
    def ReadOption(configpath, optname)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(configpath, Builtins.topath(optname)))
      )

      if optvalue == nil
        optvalue = "" # avoid nil values
      elsif Builtins.regexpmatch(optvalue, "'.*'")
        # remove the single quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "'(.*)'", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      elsif Builtins.regexpmatch(optvalue, "\".*\"")
        # remove the double quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "\"(.*)\"", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      end

      optvalue
    end


    # Read the settings for a specific LDAP server
    #
    # @param [String] server_cfgfile string filesystem path to the ldap server sysconfig file
    # @return setting map of the specified ldap server
    def ReadLdapServer(server_cfgfile)
      Builtins.y2milestone("NovellLDAP::ReadLdapServer(%1)", server_cfgfile)
      if server_cfgfile == nil || server_cfgfile == ""
        Builtins.y2error("NovellLDAP::ReadLdapServer() paramater empty!")
        return nil
      elsif Builtins.contains(@reserved_cfgfile_names, server_cfgfile)
        Builtins.y2error(
          "%1 is a not a valid ldap server cfg file name",
          server_cfgfile
        )
        return nil
      end

      # Create the SCR path to the specified LDAP server cfg file
      scr_cfg_path = Ops.add(
        Ops.add(@sysconfig_agent, path(".value")),
        Builtins.topath(
          Ops.add(Ops.add(Ops.add("\"", @config_dir), server_cfgfile), "\"")
        )
      )
      Builtins.y2debug("scr_cfg_path=%1", scr_cfg_path)

      # Read in the settings from the specified LDAP server cfg file
      ret = {}

      Ops.set(
        ret,
        "CONFIG_LDAP_SERVER_IP",
        ReadOption(scr_cfg_path, "CONFIG_LDAP_SERVER_IP")
      )
      Ops.set(
        ret,
        "CONFIG_LDAP_PORT",
        Builtins.tointeger(ReadOption(scr_cfg_path, "CONFIG_LDAP_PORT"))
      )
      Ops.set(
        ret,
        "CONFIG_LDAP_SECURE_PORT",
        Builtins.tointeger(ReadOption(scr_cfg_path, "CONFIG_LDAP_SECURE_PORT"))
      )

      Builtins.y2milestone(
        "NovellLDAP::ReadLdapServer(%1) returned: %2",
        server_cfgfile,
        ret
      )
      deep_copy(ret)
    end


    # Read all ldap server configuration settings from the config file
    #
    # @return [Array<Hash>] of all ldap servers config settings
    def ReadAllLdapServers
      ret = []

      file_list = Convert.convert(
        SCR.Read(path(".target.dir"), @config_dir),
        :from => "any",
        :to   => "list <string>"
      )
      Builtins.y2milestone("ldap_servers cfg_file_list=%1", file_list)
      if file_list != nil && Ops.greater_than(Builtins.size(file_list), 0)
        Builtins.foreach(file_list) do |server_cfgfile|
          first_char = Builtins.substring(server_cfgfile, 0, 1)
          last_char = Builtins.substring(
            server_cfgfile,
            Ops.subtract(Builtins.size(server_cfgfile), 1),
            Builtins.size(server_cfgfile)
          )
          if first_char == "." || last_char == "~"
            # ignore all files that start with '.' or end with '~'
            next
          elsif Builtins.contains(@reserved_cfgfile_names, server_cfgfile)
            # ignore files in the reserved_cfgfile_names list
            next
          else
            new_settings = ReadLdapServer(server_cfgfile)
            if new_settings != nil &&
                Ops.greater_than(Builtins.size(new_settings), 0)
              ret = Builtins.add(ret, new_settings)
            end
          end
        end
      end
      Builtins.y2milestone("NovellLdap::ReadAllLdapServers returned: %1", ret)
      deep_copy(ret)
    end


    # Read all of the OES ldap settings (including the all the ldap server sysconfig files)
    #
    # @return [Hash] of all OES LDAP settings
    def ReadLdapSettings
      ret = {}

      # Create the SCR path to the global_cfgfile
      global_cfgfile_path = Ops.add(
        path(".etc.sysconfig.novell") + path(".value"),
        Builtins.topath(@global_cfgfile)
      )
      Builtins.y2debug("global_cfgfile_path=%1", global_cfgfile_path)

      # Read in the global ldap config file settings
      Ops.set(
        ret,
        "CONFIG_LDAP_TREE_NAME",
        ReadOption(global_cfgfile_path, "CONFIG_LDAP_TREE_NAME")
      )
      Ops.set(
        ret,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ReadOption(global_cfgfile_path, "CONFIG_LDAP_ADMIN_CONTEXT")
      )

      Ops.set(
        ret,
        "CONFIG_LDAP_PROXY_CONTEXT",
        ReadOption(global_cfgfile_path, "CONFIG_LDAP_PROXY_CONTEXT")
      )

      # Context for Common Proxy
      Ops.set(
        ret,
        "CONFIG_COMMON_PROXY_CONTEXT",
        ReadOption(global_cfgfile_path, "CONFIG_COMMON_PROXY_CONTEXT")
      )

      # Since the move-to script may have been run, then the value in the sysconfig file may be out of date.
      # Only do this if the proxy user has already been configured.
      info = NovellAddonInstall.GetProductInfo("edirectory")
      if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
        currentProxyUser = NovellUtils.RetrieveProxyCredentials("username")
        Builtins.y2milestone(
          "NovellLDAP.ReadLdapSettings:checking oescredstore for proxy user since eDir is configured. User from oescredstore is '%1'",
          currentProxyUser
        )
        if currentProxyUser != nil && currentProxyUser != ""
          Ops.set(ret, "CONFIG_LDAP_PROXY_CONTEXT", currentProxyUser)
        end
      end

      Ops.set(
        ret,
        "CONFIG_LDAP_DO_PROXY",
        ReadOption(global_cfgfile_path, "CONFIG_LDAP_DO_PROXY")
      )
      if Ops.get_string(ret, "CONFIG_LDAP_DO_PROXY", "") == nil ||
          Ops.get_string(ret, "CONFIG_LDAP_DO_PROXY", "") == ""
        Ops.set(ret, "CONFIG_LDAP_DO_PROXY", "yes")
      end
      Ops.set(ret, "CONFIG_LDAP_DO_PROXY", "yes")

      Ops.set(
        ret,
        "CONFIG_LDAP_ASSIGN_POLICY",
        ReadOption(global_cfgfile_path, "CONFIG_LDAP_ASSIGN_POLICY")
      )

      Ops.set(
        ret,
        "XAD_TREE_ADMIN_CONTEXT",
        ReadOption(global_cfgfile_path, "XAD_TREE_ADMIN_CONTEXT")
      )

      # read all the ldap servers config files
      Ops.set(ret, "CONFIG_LDAP_SERVERS_LIST", ReadAllLdapServers())
      Ops.set(
        ret,
        "CONFIG_LDAP_SAVE_SERVERS_LIST",
        Ops.get_list(ret, "CONFIG_LDAP_SERVERS_LIST", [])
      )

      Builtins.y2milestone("NovellLDAP::ReadLdapSettings returned: %1", ret)
      deep_copy(ret)
    end


    # Read all oes-ldap settings
    # @return true on success
    def Read
      # NovellLDAP read dialog caption
      caption = Builtins.sformat(
        _("Initializing %1 Configuration"),
        @product_name
      )

      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          Builtins.sformat(_("Read %1 Configuration"), @product_name)
        ],
        [
          # Progress step 1/1
          Builtins.sformat(_("Reading %1 Configuration..."), @product_name),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read NovellLDAP settings
      return false if Abort()
      Progress.NextStage

      # Make sure all of the product dependencies are correct
      module_name = NovellAddonInstall.GetYaSTClient(@global_cfgfile)
      deps_correct = NovellAddonInstall.SolveProductDependencies(module_name)
      if !deps_correct
        Builtins.y2error(
          "%1 product dependencies are not correct. Aborting...",
          module_name
        )
        # FIXME: do we need a popup error for this condition???
        return false
      end

      # Read in the ldap servers configuration...
      @Settings = ReadLdapSettings() if @Settings == nil || @Settings == {}

      return false if Abort()
      # Progress finished
      Progress.NextStage
      Builtins.sleep(sl)

      @globals_configured = true
      return false if Abort()
      @modified = false
      true
    end


    # Write a setting (option) to a specified config file
    #
    # @param [Yast::Path] configpath scr path to the config file
    # @param [String] optname string key of the setting(option) we are writing
    # @param [String] optvalue string value of the setting(option) we are writing
    # @return [Boolean] true on success
    def WriteOption(configpath, optname, optvalue)
      if optvalue == nil
        optvalue = "" # avoid passing a nil value
      end

      ret = SCR.Write(Ops.add(configpath, Builtins.topath(optname)), optvalue)

      ret
    end


    # Write the settings for a specific LDAP server
    #
    # @param [Hash] ldap_server map of ldap server settings
    # @return [Boolean] true on success
    def WriteLdapServer(ldap_server)
      ldap_server = deep_copy(ldap_server)
      Builtins.y2milestone("NovellLDAP::WriteLdapServer(%1)", ldap_server)
      if ldap_server == nil || Ops.less_or_equal(Builtins.size(ldap_server), 0)
        Builtins.y2error("NovellLDAP::WriteLdapServer() paramater empty!")
        return false
      end

      server_cfgfile = Ops.get_string(ldap_server, "CONFIG_LDAP_SERVER_IP", "")

      # This code added to work-around the missing IP issue mentioned in bugzilla #712071.
      if server_cfgfile == nil || server_cfgfile == ""
        Builtins.y2error(
          "lth:'%1' is an empty ldap server cfg file name. Skipping Write.",
          server_cfgfile
        )
        return true
      end

      if Builtins.contains(@reserved_cfgfile_names, server_cfgfile)
        Builtins.y2error(
          "'%1' is a not a valid ldap server cfg file name",
          server_cfgfile
        )
        return false
      end

      # Create the SCR path to the specified LDAP server cfg file
      scr_cfg_path = Ops.add(
        Ops.add(@sysconfig_agent, path(".value")),
        Builtins.topath(
          Ops.add(Ops.add(Ops.add("\"", @config_dir), server_cfgfile), "\"")
        )
      )
      Builtins.y2milestone("scr_cfg_path=%1", scr_cfg_path)

      # Write out the settings from the specified LDAP server settings
      ret = true

      ret = ret &&
        WriteOption(
          scr_cfg_path,
          "CONFIG_LDAP_SERVER_IP",
          Ops.get_string(ldap_server, "CONFIG_LDAP_SERVER_IP", "")
        )
      ret = ret &&
        WriteOption(
          scr_cfg_path,
          "CONFIG_LDAP_PORT",
          Builtins.tostring(
            Ops.get_integer(ldap_server, "CONFIG_LDAP_PORT", 389)
          )
        )
      ret = ret &&
        WriteOption(
          scr_cfg_path,
          "CONFIG_LDAP_SECURE_PORT",
          Builtins.tostring(
            Ops.get_integer(ldap_server, "CONFIG_LDAP_SECURE_PORT", 636)
          )
        )
      ret = ret && SCR.Write(@sysconfig_agent, nil)

      Builtins.y2milestone(
        "NovellLDAP::WriteLdapServer(%1) returned: %2",
        ldap_server,
        ret
      )
      ret
    end


    # Write all ldap server configuration settings from the config file
    #
    # @param [Array<Hash>] ldap_servers_list list<map> of all ldap servers config settings
    # @return [Boolean] true on success
    def WriteAllLdapServers(ldap_servers_list)
      ldap_servers_list = deep_copy(ldap_servers_list)
      ret = true

      Builtins.foreach(ldap_servers_list) do |ldap_server|
        if ldap_server != nil && Ops.greater_than(Builtins.size(ldap_server), 0)
          ret = ret && WriteLdapServer(ldap_server)
        end
      end

      Builtins.y2milestone("NovellLdap::WriteAllLdapServers returned: %1", ret)
      ret
    end


    # Remove all deleted ldap server configuration settings from the config file
    #
    # @param [Array<Hash>] ldap_servers_list list<map> of all ldap servers config settings
    # @param ldap_save servers_list list<map> of original ldap servers config settings
    # @return [Boolean] true on success
    def RemoveDeletedLdapServers(ldap_servers_list, ldap_save_servers_list)
      ldap_servers_list = deep_copy(ldap_servers_list)
      ldap_save_servers_list = deep_copy(ldap_save_servers_list)
      ret = true
      addresses = GetAddresses(ldap_servers_list)
      old_addresses = GetAddresses(ldap_save_servers_list)
      Builtins.foreach(old_addresses) do |old_address|
        removeit = true
        Builtins.foreach(addresses) do |address|
          removeit = false if address == old_address
        end
        if removeit
          success = true
          # Don't cause a false error return if the file is already deleted.
          if SCR.Read(path(".target.size"), Ops.add(@config_dir, old_address)) != -1
            success = Convert.to_boolean(
              SCR.Execute(
                path(".target.remove"),
                Ops.add(@config_dir, old_address)
              )
            )
            Builtins.y2milestone(
              "NovellLdap::RemoveDeletedLdapServers return: %1 file: %2",
              success,
              Ops.add(@config_dir, old_address)
            )
            ret = false if !success
          else
            Builtins.y2milestone(
              "NovellLdap::RemoveDeletedLdapServers:skipping delete of file since it is already deleted. file: %1",
              Ops.add(@config_dir, old_address)
            )
          end
        end
      end
      ret
    end


    # Write all of the OES ldap settings (including the all the ldap server sysconfig files)
    #
    # @param [Hash] settings_map map of all OES ldap settings
    # @return [Boolean] true on success
    def WriteLdapSettings(settings_map)
      settings_map = deep_copy(settings_map)
      ret = true

      # Create the SCR path to the global_cfgfile
      global_cfgfile_path = Ops.add(
        path(".etc.sysconfig.novell") + path(".value"),
        Builtins.topath(@global_cfgfile)
      )
      Builtins.y2debug("global_cfgfile_path=%1", global_cfgfile_path)

      # Write out the global ldap config file settings
      ret = ret && WriteOption(global_cfgfile_path, "SERVICE_CONFIGURED", "no")
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_LDAP_TREE_NAME",
          Ops.get_string(settings_map, "CONFIG_LDAP_TREE_NAME", "")
        )
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          NovellUtils.Nds2Ldap(
            Ops.get_string(settings_map, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
        )
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "XAD_TREE_ADMIN_CONTEXT",
          NovellUtils.Nds2Ldap(
            Ops.get_string(settings_map, "XAD_TREE_ADMIN_CONTEXT", "")
          )
        )
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_LDAP_PROXY_CONTEXT",
          NovellUtils.Nds2Ldap(
            Ops.get_string(settings_map, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
        )
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_LDAP_DO_PROXY",
          Ops.get_string(settings_map, "CONFIG_LDAP_DO_PROXY", "")
        )
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_LDAP_ASSIGN_POLICY",
          Ops.get_string(settings_map, "CONFIG_LDAP_ASSIGN_POLICY", "")
        )
      ret = ret && WriteOption(global_cfgfile_path, "SERVICE_CONFIGURED", "yes")
      ret = ret && SCR.Write(path(".etc.sysconfig.novell"), nil)

      # Common Proxy Context
      ret = ret &&
        WriteOption(
          global_cfgfile_path,
          "CONFIG_COMMON_PROXY_CONTEXT",
          NovellUtils.Nds2Ldap(
            Ops.get_string(settings_map, "CONFIG_COMMON_PROXY_CONTEXT", "")
          )
      )

      # Remove deleted servers
      ret = ret &&
        RemoveDeletedLdapServers(
          Ops.get_list(settings_map, "CONFIG_LDAP_SERVERS_LIST", []),
          Ops.get_list(settings_map, "CONFIG_LDAP_SAVE_SERVERS_LIST", [])
        )
      # Write all the ldap servers config files
      ret = ret &&
        WriteAllLdapServers(
          Ops.get_list(settings_map, "CONFIG_LDAP_SERVERS_LIST", [])
        )

      Builtins.y2milestone("NovellLDAP::WriteLdapSettings returned: %1", ret)
      ret
    end

    # Get the remote servers in the current list of LDAP servers
    #
    # @param server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return list<map> remote servers
    def GetRemoteServers(server_list)
      server_list = deep_copy(server_list)
      remote_servers = []
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil

      local_addresses = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      hostname = NovellUtils.GetHostnameFQDN
      if hostname != "" || hostname != nil
        local_addresses = Ops.add(
          local_addresses,
          NovellUtils.CleanUpString(hostname)
        )
      end
      remote_servers = Builtins.filter(sl) do |server_info|
        !Builtins.contains(
          local_addresses,
          Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
        )
      end
      Builtins.y2milestone("remote_servers: %1", remote_servers)
      deep_copy(remote_servers)
    end

    # Get the info map of an address in the list
    # of LDAP servers.
    # @param [String] address The address to check
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return [Hash] for the address, or $[] if the address does
    # not exist in the list
    def GetAddressServer(address, server_list)
      server_list = deep_copy(server_list)
      ret = {}
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil

      Builtins.foreach(sl) do |server_info|
        if address == Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
          ret = deep_copy(server_info)
          raise Break
        end
      end 

      deep_copy(ret)
    end


    # mtc
    # Get the local servers in the current list of LDAP servers
    #
    # @param server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return list<map> local servers
    def GetLocalServers(server_list)
      server_list = deep_copy(server_list)
      local_servers = []
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      Builtins.y2milestone(
        "lth:GetLocalServers:CONFIG_LDAP_SERVERS_LIST: %1",
        sl
      )

      local_addresses = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      Builtins.y2milestone(
        "lth:GetLocalServers:local static IP addresses are: %1",
        local_addresses
      )

      local_servers = Builtins.filter(sl) do |server_info|
        Builtins.contains(
          local_addresses,
          Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
        )
      end

      deep_copy(local_servers)
    end




    # Function to check whether local LDAP server is running or not
    def IsLocalLDAPRunning
      cmd = "pgrep ndsd"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
      rval
    end

    # Function to get the Local LDAP ip address if LDAP server is running otherwise get the remote ip address
    def BrowseButtonLdapServer(address)
      browserIP = ""
      rval = IsLocalLDAPRunning()
      server_list = []
      primary_server = {}
      if rval != 0
        server_list = GetRemoteServers(nil)
        if Ops.greater_than(Builtins.size(server_list), 0)
          browserIP = Ops.get_string(
            Ops.get(server_list, 0, {}),
            "CONFIG_LDAP_SERVER_IP",
            ""
          )
          primary_server = GetAddressServer(browserIP, server_list)
        end
      else
        server_list = GetLocalServers(nil)
        browserIP = Ops.get_string(
          Ops.get(server_list, 0, {}),
          "CONFIG_LDAP_SERVER_IP",
          ""
        )
        primary_server = GetAddressServer(browserIP, server_list)
      end
      deep_copy(primary_server)
    end

    def Write

      # NovellLDAP write dialog caption
      caption = Builtins.sformat(_("Saving %1 Configuration"), @product_name)

      steps = 1

      sl = 500
      Builtins.sleep(sl)
      if !NovellUtils.channelUpgrade
        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          " ",
          steps,
          [
            # Progress stage 1/1
            _("Write the settings")
          ],
          [
            # Progress step 1/1
            _("Writing the settings..."),
            # Progress finished
            _("Finished")
          ],
          ""
        )  
      end
      # write settings
      return false if Abort()
      Progress.NextStage

      # Write out all the OES LDAP settings...
      success = WriteLdapSettings(@Settings)
      if success == false
        Builtins.y2error("NovellLDAP::WriteLdapSettings() failed!")
        Report.Error(
          Builtins.sformat(_("Failed to write the %1 settings"), @product_name)
        )
        return false
      end

      # Error message
      Report.Error(_("Cannot write settings.")) if false
      Builtins.sleep(sl)

      return false if Abort()
      # Progress finished
      Progress.NextStage
      #Progress.Finish
      Builtins.sleep(sl)
      return false if Abort()
      if !NovellUtils.channelUpgrade
        Progress.Finish
      end
      true
    end


    # Get all oes-ldap settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "oes-ldap", false)
      end

      # read general LDAP info
      Ops.set(
        @Settings,
        "CONFIG_LDAP_TREE_NAME",
        Ops.get_string(settings, "tree_name", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        Ops.get_string(settings, "admin_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        Ops.get_string(settings, "admin_password", "")
      )

      Ops.set(
        @Settings,
        "CONFIG_LDAP_DO_PROXY",
        Ops.get_string(settings, "use_common_proxy", "")
      )

      #Store the common proxy context (eg. o=microfocus)
      Ops.set(
        @Settings,
        "CONFIG_COMMON_PROXY_CONTEXT",
        Ops.get_string(settings, "common_proxy_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LDAP_PROXY_CONTEXT",
        "cn=OESCommonProxy_#{NovellUtils.GetHostname},#{NovellUtils.Nds2Ldap(Ops.get_string(settings, "common_proxy_context", ""))}"
      )

      Ops.set(
        @Settings,
        "CONFIG_LDAP_PROXY_PASSWORD",
        NovellUtils.GenerateRandomPassword
      )

      Ops.set(
        @Settings,
        "XAD_TREE_ADMIN_CONTEXT",
        Ops.get_string(settings, "xad_tree_admin_context", "")
      )
      Ops.set(
        @Settings,
        "XAD_TREE_ADMIN_PASSWORD",
        Ops.get_string(settings, "xad_tree_admin_password", "")
      )
      # read the server list
      server_list = []
      Builtins.foreach(Ops.get_list(settings, "ldap_servers", [])) do |server_info|
        import_server = {
          "CONFIG_LDAP_SERVER_IP"   => Ops.get_string(
            server_info,
            "ip_address",
            ""
          ),
          "CONFIG_LDAP_PORT"        => Ops.get_integer(
            server_info,
            "ldap_port",
            389
          ),
          "CONFIG_LDAP_SECURE_PORT" => Ops.get_integer(
            server_info,
            "ldaps_port",
            636
          )
        }
        server_list = Builtins.add(server_list, import_server)
      end 

      Ops.set(@Settings, "CONFIG_LDAP_SERVERS_LIST", server_list)

      @globals_configured = true

      true
    end

    # Dump the oes-ldap settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-edirectory") == false && 
                         NovellUtils.CloneSettingstoAutoinstXml("novell-imanager") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "oes-ldap", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "oes-ldap", true)
        # write the general LDAP info
        Ops.set(
          settings,
          "tree_name",
          Ops.get_string(@Settings, "CONFIG_LDAP_TREE_NAME", "")
        )
        Ops.set(
          settings,
          "admin_context",
          Ops.get_string(@Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        Ops.set(settings, "admin_password", NovellUtils.passwordMessage)

        Ops.set(
          settings,
          "use_common_proxy",
          Ops.get_string(@Settings, "CONFIG_LDAP_DO_PROXY", "")
        )
        Ops.set(
          settings,
          "common_proxy_context",
          Ops.get_string(@Settings, "CONFIG_COMMON_PROXY_CONTEXT", "")
        )
        Ops.set(
          settings,
          "xad_tree_admin_context",
          Ops.get_string(@Settings, "XAD_TREE_ADMIN_CONTEXT", "")
        )
        Ops.set(
          settings,
          "xad_tree_admin_password",
          NovellUtils.passwordMessage
        )
        # write the server list info
        server_list = []
        Builtins.foreach(
          Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", [])
        ) do |server_info|
          export_server = {
            "ip_address" => Ops.get_string(
              server_info,
              "CONFIG_LDAP_SERVER_IP",
              ""
            ),
            "ldap_port"  => Ops.get_integer(
              server_info,
              "CONFIG_LDAP_PORT",
              389
            ),
            "ldaps_port" => Ops.get_integer(
              server_info,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          }
          server_list = Builtins.add(server_list, export_server)
        end 


        Ops.set(settings, "ldap_servers", server_list)
      end

      deep_copy(settings)
    end

    # Dump the oes-ldap settings to a single map
    # (For use by proposal.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def ExportSettings
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-edirectory") == false && 
                         NovellUtils.CloneSettingstoAutoinstXml("novell-imanager") == false

      if Ops.get(NovellAddonInstall.products_to_configure, "oes-ldap", true)
        # write the general LDAP info
        Ops.set(
          settings,
          "tree_name",
          Ops.get_string(@Settings, "CONFIG_LDAP_TREE_NAME", "")
        )
        Ops.set(
          settings,
          "admin_context",
          Ops.get_string(@Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        Ops.set(
          settings,
          "admin_password",
          Ops.get_string(@Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
        )

        Ops.set(
          settings,
          "use_common_proxy",
          Ops.get_string(@Settings, "CONFIG_LDAP_DO_PROXY", "")
        )
        Ops.set(
          settings,
          "common_proxy_context",
          Ops.get_string(@Settings, "CONFIG_COMMON_PROXY_CONTEXT", "")
        )

        Ops.set(
          settings,
          "xad_tree_admin_context",
          Ops.get_string(@Settings, "XAD_TREE_ADMIN_CONTEXT", "")
        )
        Ops.set(
          settings,
          "xad_tree_admin_password",
          Ops.get_string(@Settings, "XAD_TREE_ADMIN_PASSWORD", "")
        )
        # write the server list info
        server_list = []
        Builtins.foreach(
          Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", [])
        ) do |server_info|
          export_server = {
            "ip_address" => Ops.get_string(
              server_info,
              "CONFIG_LDAP_SERVER_IP",
              ""
            ),
            "ldap_port"  => Ops.get_integer(
              server_info,
              "CONFIG_LDAP_PORT",
              389
            ),
            "ldaps_port" => Ops.get_integer(
              server_info,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          }
          server_list = Builtins.add(server_list, export_server)
        end 


        Ops.set(settings, "ldap_servers", server_list)
      end

      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)
        Builtins.foreach(
          Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", [])
        ) do |server_info|
          summary = Summary.AddListItem(
            summary,
            Builtins.sformat(
              _("LDAP Server Address: %1"),
              Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
            )
          )
        end 


        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end


    # Check if an address already exists in the current list
    # of LDAP servers.
    # @param [String] address The address to check
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return true if the address exists in the server list
    def AddressExists(address, server_list)
      server_list = deep_copy(server_list)
      ret = false
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      Builtins.foreach(sl) do |server_info|
        if Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "") != nil &&
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "") == address
          ret = true
          raise Break
        end
      end 

      ret
    end


    # Check if any local static addresses already
    # exist in the current list of LDAP servers.
    #
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return true if any local static addresses
    # exist in the server list
    def LocalAddressExists(server_list)
      server_list = deep_copy(server_list)
      ret = false
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      local_addresses = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      Builtins.foreach(sl) do |server_info|
        if Builtins.contains(
            local_addresses,
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
          )
          ret = true
          raise Break
        end
      end 

      ret
    end


    # Check if any remote addresses already
    # exist in the current list of LDAP servers.
    #
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return true if any remote addresses exist in the
    # server list
    def RemoteAddressExists(server_list)
      server_list = deep_copy(server_list)
      ret = false
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      local_addresses = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      hostname = NovellUtils.GetHostnameFQDN
      if hostname != "" || hostname != nil
        local_addresses = Ops.add(
          local_addresses,
          NovellUtils.CleanUpString(hostname)
        )
      end
      Builtins.foreach(sl) do |server_info|
        if !Builtins.contains(
            local_addresses,
            Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
          )
          ret = true
          raise Break
        end
      end 

      ret
    end

    # Convert a server list<map> into an address list<string>
    #
    # @param server_list The list of servers to convert (use nil
    # to check the currently stored list)
    # @return list<map> local servers
    def GetAddresses(server_list)
      server_list = deep_copy(server_list)
      addresses = []
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil

      Builtins.foreach(sl) do |server_info|
        addresses = Builtins.add(
          addresses,
          Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
        )
      end 

      deep_copy(addresses)
    end

    # Check if an address is local
    #
    # @param [String] address The address to check
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return true if the address exists in the server list
    # and is a local address
    def IsLocalAddress(address, server_list)
      server_list = deep_copy(server_list)
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      local_list = GetAddresses(GetLocalServers(sl))
      Builtins.contains(local_list, address)
    end


    # Check if an address is remote
    #
    # @param [String] address The address to check
    # @param [Array<Hash>] server_list The list of servers to check (use nil
    # to check the currently stored list)
    # @return true if the address exists in the server list
    # and is a remote address
    def IsRemoteAddress(address, server_list)
      server_list = deep_copy(server_list)
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      remote_list = GetAddresses(GetRemoteServers(sl))
      Builtins.contains(remote_list, address)
    end


    # Remove a server from the list
    #
    # @param addresses list of addresses to remove
    # @param server_list The list of servers to remove from
    # (use nil to check the currently stored list)
    def RemoveServersByAddress(addresses, server_list)
      addresses = deep_copy(addresses)
      server_list = deep_copy(server_list)
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      Builtins.foreach(addresses) { |address| sl = Builtins.filter(sl) do |server_info|
        address != Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
      end } 

      deep_copy(sl)
    end

    # Add a server to the list
    #
    # @param serverIP addresses to add
    # @param ldapPort ldap port to add
    # @param ldapSecurePort ldap secure port to add
    # @param server_list The list of servers to add to
    # (use nil to check the currently stored list)
    def AddServerToList(serverIP, ldapPort, ldapSecurePort, server_list)
      server_list = deep_copy(server_list)
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil

      # Put server in list if not already there
      alreadyInList = GetAddressServer(serverIP, sl)
      if alreadyInList == {}
        new_server = {
          "CONFIG_LDAP_SERVER_IP"   => serverIP,
          "CONFIG_LDAP_PORT"        => ldapPort,
          "CONFIG_LDAP_SECURE_PORT" => ldapSecurePort
        }
        sl = Builtins.add(sl, new_server)
        if server_list == nil
          Ops.set(@Settings, "CONFIG_LDAP_SERVERS_LIST", sl)
        else
          server_list = deep_copy(sl)
        end
      end
      deep_copy(sl)
    end

    # Get a list of fallback servers
    #
    # @param address of primary server
    # @param server_list The list of servers (use nil to
    # check the currently stored list)
    # @return list of fallback servers
    def GetFallbackServerList(address, server_list)
      server_list = deep_copy(server_list)
      sl = deep_copy(server_list)
      sl = Ops.get_list(@Settings, "CONFIG_LDAP_SERVERS_LIST", []) if sl == nil
      RemoveServersByAddress([address], sl)
    end

    publish :function => :GetAddresses, :type => "list <string> (list <map>)"
    publish :variable => :product_name, :type => "string"
    publish :variable => :sysconfig_agent, :type => "path"
    publish :variable => :config_dir, :type => "string"
    publish :variable => :global_cfgfile, :type => "string"
    publish :variable => :template_cfg_name, :type => "string"
    publish :variable => :template_cfgfile, :type => "string"
    publish :variable => :reserved_cfgfile_names, :type => "list <string>"
    publish :variable => :Settings, :type => "map"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Info, :type => "map <string, any> ()"
    publish :function => :NeedLDAP, :type => "boolean ()"
    publish :function => :CheckLDAPBlocker, :type => "boolean (boolean)"
    publish :function => :ReadLdapServer, :type => "map (string)"
    publish :function => :ReadAllLdapServers, :type => "list <map> ()"
    publish :function => :ReadLdapSettings, :type => "map ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :WriteLdapServer, :type => "boolean (map)"
    publish :function => :WriteAllLdapServers, :type => "boolean (list <map>)"
    publish :function => :RemoveDeletedLdapServers, :type => "boolean (list <map>, list <map>)"
    publish :function => :WriteLdapSettings, :type => "boolean (map)"
    publish :function => :GetRemoteServers, :type => "list <map> (list <map>)"
    publish :function => :GetAddressServer, :type => "map (string, list <map>)"
    publish :function => :GetLocalServers, :type => "list <map> (list <map>)"
    publish :function => :IsLocalLDAPRunning, :type => "integer ()"
    publish :function => :BrowseButtonLdapServer, :type => "map (string)"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ExportSettings, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
    publish :function => :AddressExists, :type => "boolean (string, list <map>)"
    publish :function => :LocalAddressExists, :type => "boolean (list <map>)"
    publish :function => :RemoteAddressExists, :type => "boolean (list <map>)"
    publish :function => :IsLocalAddress, :type => "boolean (string, list <map>)"
    publish :function => :IsRemoteAddress, :type => "boolean (string, list <map>)"
    publish :function => :RemoveServersByAddress, :type => "list <map> (list <string>, list <map>)"
    publish :function => :AddServerToList, :type => "list <map> (string, integer, integer, list <map>)"
    publish :function => :GetFallbackServerList, :type => "list <map> (string, list <map>)"
    publish :variable => :auto_clone, :type => "boolean"
  end

  NovellLDAP = NovellLDAPClass.new
  NovellLDAP.main
end
