# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	modules/OesLdap.ycp
# Package:	Configuration of oes-ldap
# Summary:	OesLdap settings, input and output functions
# Authors:	Novell YaST2 Development Team <support@novell.com>
#
# $Id: OesLdap.ycp,v 1.1.2.6 2005/10/05 16:24:04 rhaslam Exp $
#
# Representation of the configuration of oes-ldap.
# Input and output routines.
require "yast"

module Yast
  class OesLdapClass < Module
    def main
      Yast.import "UI"
      textdomain "oes-ldap"

      Yast.import "EdirUtils"
      Yast.import "Label"
      Yast.import "Mode"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "Progress"
      Yast.import "Popup"
      Yast.import "Report"
      Yast.import "String"
      Yast.import "Summary"
      Yast.import "ValidationHelpers"
      Yast.import "Wizard"
      Yast.import "NovellAddonInstall"

      Yast.include self, "oes-ldap/helps.rb"

      @lastSaved = {}
      @autoyast_tool = false
      @LDAPSettings = {}
      @LOCAL_HOST = _("Local Host")
      @local_server = true

      # Data was modified?
      @modified = false

      # is the proposal valid?
      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end


    # SetLDAPEdirSettings (map settings)
    #
    # Called from eDirectory dialogs to set initial data for the autoyast tool
    #
    # @param map of eDirectory LDAP settings
    # @return [Boolean] true on success
    def SetLDAPEdirSettingsinLastSaved(settings)
      settings = deep_copy(settings)
      ret = false
      # If lastSaved is empty, then we will load it initial from eDirectory.
      if @autoyast_tool == true && @lastSaved == {}
        Builtins.y2milestone(
          "SetLDAPEdirSettingsinLastSaved() Loading lastSavedValues from eDirectory the first time."
        )

        Ops.set(
          @lastSaved,
          "CONFIG_EDIR_TREE_TYPE",
          Ops.get_string(settings, "CONFIG_EDIR_TREE_TYPE", "")
        )

        treeType = Ops.get_string(@lastSaved, "CONFIG_EDIR_TREE_TYPE", "")
        if treeType == "existing"
          Ops.set(@lastSaved, "CONFIG_LDAP_SERVER_IP", _("Local Host"))
        else
          Ops.set(
            @lastSaved,
            "CONFIG_LDAP_SERVER_IP",
            Ops.get_string(settings, "CONFIG_LDAP_SERVER_IP", "")
          )
        end

        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_PORT",
          Ops.get_integer(settings, "CONFIG_LDAP_PORT", 389)
        )
        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_SECURE_PORT",
          Ops.get_integer(settings, "CONFIG_LDAP_SECURE_PORT", 636)
        )
        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          Ops.get_string(settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          Ops.get_string(settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
        )
        Ops.set(
          @lastSaved,
          "CONFIG_EDIR_TREE_NAME",
          Ops.get_string(settings, "CONFIG_EDIR_TREE_NAME", "")
        )
        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_PROXY_CONTEXT",
          Ops.get_string(settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
        )
        Ops.set(
          @lastSaved,
          "CONFIG_LDAP_DO_PROXY",
          Ops.get_string(settings, "CONFIG_LDAP_DO_PROXY", "")
        )
        ret = true
      end
      ret
    end


    # ReadLDAPSettingsForServer (string ipAddr)
    #
    # Reads data from the file /etc/sysconfig/novell/ldap_defaults
    #
    # If IP_ADDRESS == "" then the local IP address is used
    #
    # @param string ip address
    # @return [Hash] of LDAP settings assosiated is the sepecified address
    def ReadLDAPSettingsForServer(ipAddr)
      Builtins.y2milestone("OesLdap::ReadLDAPSettingsForServer(%1)", ipAddr)
      #if the global settings are not empty return them, otherwise read them in.
      if @LDAPSettings != {}
        Builtins.y2milestone("LDAP Settings already exist!")
        return deep_copy(@LDAPSettings)
      end

      Builtins.y2milestone(
        "Reading LDAP data from file /etc/sysconfig/novell/ldap_defaults"
      )
      # Tree Name IF eDirectory based LDAP
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_TREE_NAME"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_TREE_NAME", val)

      # Admin Context
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_ADMIN_CONTEXT"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_ADMIN_CONTEXT", val)

      # Proxy Context
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_PROXY_CONTEXT"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_PROXY_CONTEXT", val)

      # Use common proxy
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_DO_PROXY"))
      )
      val = "yes" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_DO_PROXY", val)

      # Server Context (if eDirectory)
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVER_CONTEXT"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_SERVER_CONTEXT", val)

      # Local IP
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_LOCAL_IP"))
      )
      val = NovellUtils.GetIPAddr if val == nil || val == ""
      Ops.set(@LDAPSettings, "CONFIG_LDAP_LOCAL_IP", val)

      # Server IP
      # FIXME: can this value go away....why not use the 1st item in the server IP list???
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVER_IP"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      Ops.set(@LDAPSettings, "CONFIG_LDAP_SERVER_IP", val)

      # LDAP Port
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_PORT"))
      )
      val = "389" if val == nil || val == nil
      Ops.set(@LDAPSettings, "CONFIG_LDAP_PORT", Builtins.tointeger(val))

      # LDAP Secure Port
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_SECURE_PORT"))
      )
      val = "636" if val == nil || val == nil
      Ops.set(@LDAPSettings, "CONFIG_LDAP_SECURE_PORT", Builtins.tointeger(val))


      # LDAP Servers list
      val = Convert.to_string(
        SCR.Read(path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVERS_LIST"))
      )
      val = "" if val == nil # prevent a nil value in the LDAPSettings map
      # parse the cfg string and generate a list<string> of server ip addresses
      serverDataList = Builtins.splitstring(val, ";")
      serverDataList = Builtins.filter(serverDataList) do |f|
        f != nil && f != ""
      end # remove empty list items
      serverDataList = Builtins.maplist(serverDataList) do |v|
        String.CutBlanks(v)
      end # trim white space around values
      serverDataList = Builtins.toset(serverDataList) # remove duplicate entries

      ldapServerList = []
      Builtins.foreach(serverDataList) do |serverDataString|
        server_data = Builtins.splitstring(serverDataString, ",")
        server_data = Builtins.filter(server_data) { |f| f != nil && f != "" } # remove empty list items
        server_data = Builtins.maplist(server_data) { |v| String.CutBlanks(v) } # trim white space around values
        server_data = Builtins.toset(server_data) # remove duplicate entries
        ldapServerList = Builtins.add(ldapServerList, server_data)
      end

      if Builtins.size(ldapServerList) == 0 &&
          Ops.get_string(@LDAPSettings, "CONFIG_LDAP_SERVER_IP", "") != ""
        lport = Builtins.sformat(
          "%1",
          Ops.get_integer(@LDAPSettings, "CONFIG_LDAP_PORT", 389)
        )
        lsport = Builtins.sformat(
          "%1",
          Ops.get_integer(@LDAPSettings, "CONFIG_LDAP_SECURE_PORT", 636)
        )
        l = [
          Ops.get_string(@LDAPSettings, "CONFIG_LDAP_SERVER_IP", ""),
          lport,
          lsport
        ]
        ldapServerList = Builtins.add(ldapServerList, l)
      end
      Ops.set(@LDAPSettings, "CONFIG_LDAP_SERVERS_LIST", ldapServerList) # FIXME: why are we bothering to keep a list or server IPs?  It was never written properly to the ldap_defaults cfg and it is not used by any module????

      deep_copy(@LDAPSettings)
    end


    # SaveLDAPSettingsForServer (string ipAddr)
    #
    # Saves data to the file /etc/sysconfig/novell/ldap_defaults
    #
    # If IP_ADDRESS == "" then the local IP address is used
    #
    # @param string ip address
    # @param map LDAP settings assosiate with the given IP address
    # @return [Boolean] true on success
    def SaveLDAPSettingsForServer(ipAddr, data)
      data = deep_copy(data)
      if ipAddr == ""
        ipAddr = NovellUtils.GetIPAddr # FIXME: this is bogus...ipAddr is never used after this!
      end
      Builtins.y2debug("Writing ldap_defaults for %1 content %2", ipAddr, data)

      wok = false
      SCR.Write(path(".sysconfig.ldap_defaults.SERVICE_CONFIGURED"), "no")

      # Tree Name IF eDirectory based LDAP
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_TREE_NAME"),
        Ops.get_string(data, "CONFIG_LDAP_TREE_NAME", "")
      )

      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "TREE_NAME")
        return wok
      end

      # Admin Context
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_ADMIN_CONTEXT"),
        Ops.get_string(data, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )

      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "ADMIN_CONTEXT")
        return wok
      end

      # Proxy Context
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_PROXY_CONTEXT"),
        Ops.get_string(data, "CONFIG_LDAP_PROXY_CONTEXT", "")
      )

      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "PROXY_CONTEXT")
        return wok
      end

      # Use common Proxy
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_DO_PROXY"),
        Ops.get_string(data, "CONFIG_LDAP_DO_PROXY", "")
      )

      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "DO_PROXY")
        return wok
      end

      # Server Context
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVER_CONTEXT"),
        Ops.get_string(data, "CONFIG_LDAP_SERVER_CONTEXT", "")
      )

      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "SERVER_CONTEXT")
        return wok
      end

      # Local IP
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_LOCAL_IP"),
        Ops.get_string(data, "CONFIG_LDAP_LOCAL_IP", "")
      )
      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "LOCAL_IP")
        return wok
      end

      # LDAP Ports
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_PORT"),
        Builtins.sformat("%1", Ops.get_integer(data, "CONFIG_LDAP_PORT", -1))
      )
      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "LDAP_PORT") # FIXME: this is bogus...the oes-ldap UI allows mapping of LDAP ports to an IP address :-(
        return wok
      end

      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_SECURE_PORT"),
        Builtins.sformat(
          "%1",
          Ops.get_integer(data, "CONFIG_LDAP_SECURE_PORT", -1)
        )
      )
      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "SECURE_PORT") # FIXME: this is bogus...the oes-ldap UI allows mapping of LDAP ports to an IP address :-(
        return wok
      end


      # LDAP Servers String
      ldapServersString = ""
      serversList = Ops.get_list(data, "CONFIG_LDAP_SERVERS_LIST", [])
      Builtins.foreach(serversList) do |server|
        # generate a comma seperated string
        formattedString = ""
        Builtins.foreach(server) do |v|
          if formattedString == ""
            formattedString = Builtins.sformat("%1", v) # sformat ensures the value is cast to string
          else
            formattedString = Builtins.sformat("%1,%2", formattedString, v) # add comma between values
          end
        end
        # generate semicolon seperated string
        if ldapServersString == ""
          ldapServersString = formattedString
        else
          ldapServersString = Builtins.sformat(
            "%1 ; %2",
            ldapServersString,
            formattedString
          )
        end
      end
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVERS_LIST"),
        ldapServersString
      ) # FIXME: why are we bothering to keep a list or server IPs?  It was never written properly to the ldap_defaults cfg and it is not used by any module????
      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "SERVERS_LIST")
        return wok
      end


      # Server IP
      wok = SCR.Write(
        path(".sysconfig.ldap_defaults.CONFIG_LDAP_SERVER_IP"),
        Ops.get_string(data, "CONFIG_LDAP_SERVER_IP", "")
      )
      if !wok
        Builtins.y2milestone("FAILED WRITE %1", "SERVER_IP")
        return wok
      end

      SCR.Write(path(".sysconfig.ldap_defaults.SERVICE_CONFIGURED"), "yes")

      # Flush the cache
      wok = SCR.Write(path(".sysconfig.ldap_defaults"), nil)
      if !wok
        Builtins.y2milestone("FAILED FLUSH")
        return wok
      end

      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_TREE_NAME",
        Ops.get_string(data, "CONFIG_LDAP_TREE_NAME", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        Ops.get_string(data, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_PROXY_CONTEXT",
        Ops.get_string(data, "CONFIG_LDAP_PROXY_CONTEXT", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_DO_PROXY",
        Ops.get_string(data, "CONFIG_LDAP_DO_PROXY", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        Ops.get_string(data, "CONFIG_LDAP_ADMIN_PASSWORD", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_SERVER_CONTEXT",
        Ops.get_string(data, "CONFIG_LDAP_SERVER_CONTEXT", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_SERVER_IP",
        Ops.get_string(data, "CONFIG_LDAP_SERVER_IP", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_LOCAL_IP",
        Ops.get_string(data, "CONFIG_LDAP_LOCAL_IP", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_PORT",
        Ops.get_integer(data, "CONFIG_LDAP_PORT", 389)
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_SECURE_PORT",
        Ops.get_integer(data, "CONFIG_LDAP_SECURE_PORT", 636)
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_SERVERS_LIST",
        Ops.get_list(data, "CONFIG_LDAP_SERVERS_LIST", [])
      )

      Builtins.y2milestone("Write LDAP Settings FINISHED")
      true
    end


    # Validate a set of LDAP settings
    #
    # @param map of LDAP settings
    # @return [Boolean] true if the LDAP settings are valid
    def ValidLDAPSettings(settings)
      settings = deep_copy(settings)
      val = ""

      # Hide the password. Bugzilla 248295
      val = Ops.get_string(settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
      if val != nil && val != ""
        Ops.set(settings, "CONFIG_LDAP_ADMIN_PASSWORD", "****")
      end
      Builtins.y2milestone("OesLdap::ValidateLDAPSettings(%1)", settings)
      Ops.set(settings, "CONFIG_LDAP_ADMIN_PASSWORD", val)

      ret = true

      val = Ops.get_string(settings, "CONFIG_LDAP_SERVER_IP", "")
      ret = false if val == nil || val == ""

      val = Ops.get_string(settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      ret = false if val == nil || val == ""

      intval = Ops.get_integer(settings, "CONFIG_LDAP_PORT", 0)
      ret = false if intval == nil || Ops.less_or_equal(intval, 0)

      intval = Ops.get_integer(settings, "CONFIG_LDAP_SECURE_PORT", 0)
      ret = false if intval == nil || intval == 0

      val = Ops.get_string(settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
      ret = false if val == nil || val == ""

      Builtins.y2milestone("OesLdap::ValidateLDAPSettings returned: %1", ret)
      ret
    end


    # Read all oes-ldap settings
    # @return true on success
    def Read
      Builtins.y2milestone("Entered OesLdap::Read()")
      # OesLdap read dialog caption
      caption = _(
        "Initializing LDAP Configuration for Open Enterprise Services"
      )

      sl = 1000

      stages = [
        _("Read the existing LDAP configuration"),
        _("Process data from the existing LDAP configuration")
      ]

      steps = Builtins.size(stages)
      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        _("Read LDAP configuration information"),
        steps,
        stages,
        stages,
        ""
      )

      Progress.NextStage

      # read database
      return false if Abort()
      @LDAPSettings = ReadLDAPSettingsForServer("")
      Progress.NextStage
      Builtins.sleep(sl)

      # process the database
      return false if Abort()

      @globals_configured = true
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      Builtins.y2debug("LDAP Settings map = %1", @LDAPSettings)
      Builtins.y2milestone("Completed OesLdap::Read()")
      return false if Abort()
      @modified = false
      true
    end


    # Write all oes-ldap settings
    # @return true on success
    def Write
      Builtins.y2milestone("OesLdap::Write started")
      Wizard.DisableAbortButton
      if !Ops.get(NovellAddonInstall.products_to_configure, "oes-ldap", true)
        Builtins.y2milestone("OES LDAP -- configure later")
        return true
      end

      Builtins.y2milestone("Entered OesLdap::Write()")
      # OesLdap read dialog caption
      caption = _("Saving LDAP Configuration for Open Enterprise Services")

      # TODO FIXME And set the right number of stages

      sl = 1000
      Builtins.sleep(sl)

      stages = [_("Writing enhanced LDAP Configuration")]
      steps = Builtins.size(stages)

      Progress.New(caption, " ", steps, stages, stages, "")
      # write settings
      return false if Abort()
      Progress.NextStage

      ret = SaveLDAPSettingsForServer("", @LDAPSettings)
      Builtins.y2milestone(
        "OesLdap::SaveLDAPSettingsForServer returned: %1",
        ret
      )
      return ret if ret == false
      Builtins.sleep(sl)

      return false if Abort()
      # Progress finished
      Progress.Title(_("Finished"))
      Progress.NextStage
      Builtins.sleep(sl)
      Builtins.y2milestone("Completed OesLdap::Write()")
      true
    end

    # Get all oes-ldap settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        return true
      end
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_TREE_NAME",
        Ops.get_string(settings, "tree_name", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        Ops.get_string(settings, "admin_context", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_PROXY_CONTEXT",
        Ops.get_string(settings, "proxy_context", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_DO_PROXY",
        Ops.get_string(settings, "use_common_proxy", "")
      )
      Ops.set(
        @LDAPSettings,
        "CONFIG_LDAP_SERVERS_LIST",
        Ops.get_string(settings, "ldap_servers", "")
      )

      ldapServers = Ops.get_string(
        @LDAPSettings,
        "CONFIG_LDAP_SERVERS_LIST",
        ""
      )
      dataList = Builtins.splitstring(ldapServers, ";")
      ldapServerList = []
      firstServer = true
      Builtins.foreach(dataList) do |i|
        oes_ldap_server = {}
        server = Builtins.splitstring(i, ",")
        if firstServer == true
          firstServer = false
          Ops.set(
            @LDAPSettings,
            "CONFIG_LDAP_PORT",
            Builtins.tointeger(Ops.get(server, 1, "-1"))
          )
          Ops.set(
            @LDAPSettings,
            "CONFIG_LDAP_SECURE_PORT",
            Builtins.tointeger(Ops.get(server, 2, "-1"))
          )
        end
        ldapServerList = Builtins.add(ldapServerList, server)
      end
      Ops.set(@LDAPSettings, "CONFIG_LDAP_SERVERS_LIST", ldapServerList)

      @globals_configured = true
      true
    end

    # Dump the oes-ldap settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}
      ldap_servers = {}
      #tree variables
      Ops.set(
        settings,
        "tree_name",
        Ops.get_string(@LDAPSettings, "CONFIG_LDAP_TREE_NAME", "")
      )
      Ops.set(
        settings,
        "admin_context",
        Ops.get_string(@LDAPSettings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      Ops.set(
        settings,
        "proxy_context",
        Ops.get_string(@LDAPSettings, "CONFIG_LDAP_PROXY_CONTEXT", "")
      )
      Ops.set(
        settings,
        "use_commmon_proxy",
        Ops.get_string(@LDAPSettings, "CONFIG_LDAP_DO_PROXY", "")
      )

      # Process the list of LDAP servers to a set of server entries
      ldapServers = Ops.get_list(@LDAPSettings, "CONFIG_LDAP_SERVERS_LIST", [])
      Builtins.foreach(ldapServers) do |i|
        server = {}
        Ops.set(server, "ldap_ip_address", Ops.get_string(i, 0, ""))
        Ops.set(server, "ldap_port", Ops.get_string(i, 1, "-1"))
        Ops.set(server, "ldap_secure_port", Ops.get_string(i, 2, "-1"))
        Builtins.add(ldap_servers, "server", server)
      end

      Ops.set(settings, "ldap_servers", ldap_servers)
      deep_copy(settings)
    end

    # Tranlate a yes or no string
    # @return translated "yes" or "no" string
    def Trans_yes_no(yesno)
      if Builtins.tolower(yesno) == "yes"
        return _("yes")
      else
        return _("no")
      end
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        # Tree Name
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Tree Name: %1"),
            Ops.get_string(@LDAPSettings, "CONFIG_LDAP_TREE_NAME", "")
          )
        )

        # Admin Name
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Admin Name: %1"),
            Ops.get_string(@LDAPSettings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
        )

        # Admin Password
        status = ""
        password = Ops.get_string(
          @LDAPSettings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        if password == ""
          status = _("Password missing")
        else
          status = _("Password available")
        end

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(_("AdminPassword: %1"), status)
        )

        # Use Common Proxy
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Use Common Proxy for product defaults: %1"),
            Trans_yes_no(
              Ops.get_string(@LDAPSettings, "CONFIG_LDAP_DO_PROXY", "")
            )
          )
        )

        # Proxy Name
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Common Proxy User Name: %1"),
            Ops.get_string(@LDAPSettings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
        )

        # Proxy Password
        status = ""
        password = Ops.get_string(
          @LDAPSettings,
          "CONFIG_LDAP_PROXY_PASSWORD",
          ""
        )
        if password == ""
          status = _("Password missing")
        else
          status = _("Password available")
        end

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(_("Proxy User Password: %1"), status)
        )

        # LDAP Servers
        summary = Summary.AddListItem(summary, _("Configured LDAP Servers:"))
        sl = Ops.get_list(@LDAPSettings, "CONFIG_LDAP_SERVERS_LIST", [])
        Builtins.foreach(sl) do |i|
          serverip = Ops.get_string(i, 0, "")
          summary = Summary.AddListItem(summary, serverip)
        end
        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end







    def ReallyAbort
      !Modified() || Popup.ReallyAbort(true)
    end


    # LDAP_Dialog_Opt
    #
    # Obtains the LDAP configuration
    # for a service. Additional parameters are provided to
    # select whether:
    #        1. A DNS name can be used for the serverIP
    #        2. The back button should be enabled on this dialog
    #
    # @param string name of the component using this common dialog
    # @param default map of settings for the LDAP dialog
    # @param [Boolean] useIPOnlyNoDNS - whether a DNS name can be used for the serverIP
    # @param [Boolean] enableBack - should the `back button of this dialog be enabled or disabled
    # @return dialog result
    def LDAP_Dialog_Opt(component, defaults, useIPOnlyNoDNS, enableBack)
      defaults = deep_copy(defaults)
      # Variable initialization
      serverIP = "0.0.0.0"
      localServerIP = NovellUtils.GetIPAddr
      results = {}
      passwordAvailable = false
      localServer = true
      localAdminContext = ""

      # Local display variables
      ldapServerIP = ""
      adminContext = ""
      ldapPort = 389
      ldapSPort = 636
      currentValues = {}

      # LDAP_Dialog caption
      caption = Builtins.sformat(_("%1 LDAP Server Configuration"), component)

      #Is a password available in the defaults
      adminPassword = Ops.get_string(defaults, "CONFIG_LDAP_ADMIN_PASSWORD", "")

      Builtins.y2debug("Values passed to LDAP_Dialog() %1", defaults)
      Builtins.y2debug("Admin password = %1", adminPassword)
      if @autoyast_tool
        if @lastSaved == {}
          Builtins.y2milestone(
            "LDAP_Dialog() Loading lastSavedValues from eDirectory the first time."
          )
        end
        Builtins.y2milestone("LDAP_Dialog() using lastSaved values")
        ldapServerIP = Ops.get_string(@lastSaved, "CONFIG_LDAP_SERVER_IP", "")
        ldapPort = Ops.get_integer(@lastSaved, "CONFIG_LDAP_PORT", 389)
        ldapSPort = Ops.get_integer(@lastSaved, "CONFIG_LDAP_SECURE_PORT", 636)
        adminContext = Ops.get_string(
          @lastSaved,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          ""
        )
        adminPassword = Ops.get_string(
          @lastSaved,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
      elsif defaults == {}
        # This component has no private defaults so we must set our own
        Builtins.y2milestone(
          "LDAP_Dialog() using values from /etc/sysconfig/novell/ldap_defaults"
        )
        # Read local eDirectory settings
        # If no file exists the Read will establish
        # default values
        currentValues = ReadLDAPSettingsForServer("")
        if Ops.greater_than(Builtins.size(currentValues), 0)
          ldapServerIP = Ops.get_string(
            currentValues,
            "CONFIG_LDAP_SERVER_IP",
            ""
          )
          ldapPort = Ops.get_integer(currentValues, "CONFIG_LDAP_PORT", 389)
          ldapSPort = Ops.get_integer(
            currentValues,
            "CONFIG_LDAP_SECURE_PORT",
            636
          )
          adminContext = Ops.get_string(
            currentValues,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            ""
          )
          localAdminContext = adminContext
          if adminPassword == ""
            passwordAvailable = false
          else
            passwordAvailable = true
          end
          localAdminPassword = adminPassword
        end
      else
        Builtins.y2milestone(
          "LDAP_Dialog() default values established from the calling component"
        )
        ldapServerIP = Ops.get_string(defaults, "CONFIG_LDAP_SERVER_IP", "")
        ldapPort = Ops.get_integer(defaults, "CONFIG_LDAP_PORT", 389)
        ldapPort = 389 if ldapPort == nil
        ldapSPort = Ops.get_integer(defaults, "CONFIG_LDAP_SECURE_PORT", 636)
        ldapSPort = 636 if ldapSPort == nil
        adminContext = Ops.get_string(defaults, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      end

      Builtins.y2milestone(
        "Initial values for LDAP dialogare: LDAP_PORT=%1\n" +
          " LDAP_SECURE_PORT=%2\n" +
          " LDAP_ADMIN_CONTEXT=%3\n",
        ldapPort,
        ldapSPort,
        adminContext
      )

      # Determine to start out with local LDAP server or
      #remote LDAP server selected
      if @autoyast_tool
        if Builtins.size(@lastSaved) == 0
          localServer = true
          @local_server = true
        else
          lastSavedServerIP = Ops.get_string(
            @lastSaved,
            "CONFIG_LDAP_SERVER_IP",
            ""
          )
          if lastSavedServerIP == "" || lastSavedServerIP == _("Local Host")
            localServer = true
            @local_server = true
          else
            localServer = false
            @local_server = false
          end
        end
      else
        if ldapServerIP != localServerIP
          localServer = false
          @local_server = false
        end
      end

      #set results in case they abort or go back we still have data.
      Ops.set(results, "CONFIG_LDAP_IP_ADDRESS", ldapServerIP)
      Ops.set(results, "CONFIG_LDAP_SERVER_IP", ldapServerIP)
      Ops.set(results, "CONFIG_LDAP_ADMIN_CONTEXT", adminContext)
      Ops.set(results, "CONFIG_LDAP_ADMIN_PASSWORD", adminPassword)

      # In case the ports are not modified we save them here
      Ops.set(results, "CONFIG_LDAP_PORT", ldapPort)
      Ops.set(results, "CONFIG_LDAP_SECURE_PORT", ldapSPort)

      ldapServerIP = _("Local Host") if localServer && @autoyast_tool

      optionalMsg = ""
      optionalMsg = _(". Optional for Desktop.") if !NovellUtils.IsServerProduct
      # LDAP dialog contents
      contents = HVSquash(
        VBox(
          Left(Label(_("Local or Remote Directory Server"))),
          RadioButtonGroup(
            Id(:mode),
            Opt(:notify),
            VBox(
              Left(
                RadioButton(
                  Id(:local),
                  Opt(:notify),
                  _("&Local system"),
                  localServer
                )
              ),
              Left(
                RadioButton(
                  Id(:remote),
                  Opt(:notify),
                  _("&Remote system"),
                  !localServer
                )
              )
            )
          ),
          VSpacing(1),
          TextEntry(
            Id(:directoryServerIP),
            _("Directory server address"),
            ldapServerIP
          ),
          TextEntry(
            Id(:adminContext),
            Ops.add(
              _("Admin name with context (i.e. cn=admin,o=microfocus)"),
              optionalMsg
            ), # BUGZILLA 102389 added spaces to compensate for GUI cutoff
            adminContext
          ),
          Password(
            Id(:adminPassword),
            Ops.add(_("Admin password"), optionalMsg),
            adminPassword
          ),
          VSpacing(2),
          PushButton(Id(:portButton), _("Port Details"))
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "novellLdap", ""),
        Label.BackButton,
        Label.NextButton
      )

      UI.ChangeWidget(Id(:back), :Enabled, enableBack)

      ret = nil
      UI.ChangeWidget(Id(:directoryServerIP), :Enabled, false) if localServer
      while true
        ret = UI.UserInput
        Ops.set(results, "RETURN_CODE", ret)

        #Abort
        if ret == :abort || ret == :cancel || ret == :back
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          # Get new values
          serverIP = Convert.to_string(
            UI.QueryWidget(Id(:directoryServerIP), :Value)
          )
          adminContext = Convert.to_string(
            UI.QueryWidget(Id(:adminContext), :Value)
          )
          adminPassword = Convert.to_string(
            UI.QueryWidget(Id(:adminPassword), :Value)
          )
          #Validate all data

          #The IP address can be blank when autoyast_tool is run
          if !localServer
            if useIPOnlyNoDNS
              next if !NovellValidators.ValidateIP(serverIP)
            elsif !NovellValidators.ValidateDNSorIP(serverIP)
              next
            end
          else
            serverIP = "" if serverIP == _("Local Host")
          end
          if NovellUtils.IsServerProduct || adminContext != ""
            if !NovellValidators.ValidateFDN(adminContext, false)
              next
            else
              adminContext = ValidationHelpers.toLdap(adminContext)
            end
          end
          if NovellUtils.IsServerProduct || adminPassword != ""
            next if !NovellValidators.ValidatePassword(adminPassword)
          end

          # Can't validate ports Edirectory has started
          # 			if (!NovellValidators::ValidatePort(ldapPort))
          # 				continue;
          # 			if (!NovellValidators::ValidatePort(ldapSPort))
          # 				continue;
          Ops.set(results, "CONFIG_LDAP_IP_ADDRESS", serverIP)
          Ops.set(results, "CONFIG_LDAP_SERVER_IP", serverIP)
          Ops.set(results, "CONFIG_LDAP_ADMIN_CONTEXT", adminContext)
          Ops.set(results, "CONFIG_LDAP_ADMIN_PASSWORD", adminPassword)

          if @autoyast_tool
            Ops.set(@lastSaved, "CONFIG_LDAP_IP_ADDRESS", serverIP)
            Ops.set(@lastSaved, "CONFIG_LDAP_SERVER_IP", serverIP)
            Ops.set(@lastSaved, "CONFIG_LDAP_ADMIN_CONTEXT", adminContext)
            Ops.set(@lastSaved, "CONFIG_LDAP_ADMIN_PASSWORD", adminPassword)
          end

          # Validate the LDAP credentials
          # Local eDirectory server should be up
          # We assume that remote LDAP servers are up

          if NovellUtils.IsServerProduct ||
              adminContext != "" && adminPassword != ""
            err = NovellValidators.ValidateCredentials(
              adminContext,
              adminPassword,
              serverIP,
              ldapSPort,
              true
            )
            if err != 0
              if err == -666
                err2 = _(
                  "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
                    "This may indicate a problem with LDAP on the target server.\n" +
                    "If you are confident that your credentials are correct you may continue,\n" +
                    "otherwise please determine if LDAP is running correctly on the target system\n" +
                    "Do you want to continue?"
                )
                yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err2)
                next if !yes
              else
                if !Popup.YesNo(
                    Builtins.sformat(
                      _(
                        "Could not authenticate via LDAP using the following parameters:\n" +
                          "   user name: %1\n" +
                          "   address:   %2\n" +
                          "   port:      %3\n" +
                          " \n" +
                          " This may indicate that one of the parameters is in error.\n" +
                          " However, if LDAP is not enabled on the eDirectory server\n" +
                          " or configured to listen on a different port than the\n" +
                          " above port, we will not get a response back.\n" +
                          " \n" +
                          " Do you want to continue and use these parameters anyway?"
                      ),
                      adminContext,
                      serverIP,
                      ldapSPort
                    )
                  )
                  next
                end
              end
            end
          end
          break
        elsif ret == :local
          localServer = true
          @local_server = true
          UI.ChangeWidget(Id(:directoryServerIP), :Enabled, false)

          localServerIP = _("Local Host") if @autoyast_tool
          UI.ChangeWidget(Id(:directoryServerIP), :Value, localServerIP)
          UI.ChangeWidget(Id(:adminContext), :Value, localAdminContext)
          UI.ChangeWidget(Id(:adminPassword), :Value, defined?(localAdminPassword).nil? ? "" : localAdminPassword)
        elsif ret == :remote
          localServer = false
          @local_server = false
          UI.ChangeWidget(Id(:directoryServerIP), :Enabled, true)
          UI.ChangeWidget(Id(:directoryServerIP), :Value, "")
          UI.ChangeWidget(Id(:adminContext), :Value, "")
          UI.ChangeWidget(Id(:adminPassword), :Value, "")
        elsif ret == :portButton
          # Read the data for the specified server if
          # available
          currentServerIP = Convert.to_string(
            UI.QueryWidget(Id(:directoryServerIP), :Value)
          )
          # validate address
          if @autoyast_tool && localServer == true
            Builtins.y2milestone(
              "OesLdap:skipping IP / hostname validation for AutoYaST tool."
            )
          else
            Builtins.y2milestone("OesLdap:Validating IP address / hostname.")
            next if !NovellValidators.ValidateDNSorIP(currentServerIP)
          end
          contents2 = HVSquash(
            VBox(
              IntField(Id(:ldapPort), _("LDAP port"), 0, 65535, ldapPort),
              IntField(
                Id(:ldapSPort),
                _("LDAP secure port"),
                0,
                65535,
                ldapSPort
              ),
              PushButton(Id(:ok), Opt(:default, :key_F10), Label.OKButton)
            )
          )
          UI.OpenDialog(Opt(:decorated), contents2)
          UI.SetFocus(Id(:ok))
          while true
            ret = UI.UserInput
            if ret == :ok
              ldapPort = Convert.to_integer(
                UI.QueryWidget(Id(:ldapPort), :Value)
              )
              Ops.set(results, "CONFIG_LDAP_PORT", ldapPort)
              ldapSPort = Convert.to_integer(
                UI.QueryWidget(Id(:ldapSPort), :Value)
              )
              Ops.set(results, "CONFIG_LDAP_SECURE_PORT", ldapSPort)
              break
            end
          end
          UI.CloseDialog
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
      Builtins.y2milestone("ND:localServer value: %1", localServer)
      deep_copy(results)
    end

    # LDAP_Dialog
    #
    # Obtains the LDAP configuration
    # for a service. The useIPOnlyNoDNS and enableBack
    # options are defaulted to false
    #
    # @param string name of the component using this common dialog
    # @param default map of settings for the LDAP dialog
    # @return dialog result
    def LDAP_Dialog(component, defaults)
      defaults = deep_copy(defaults)
      LDAP_Dialog_Opt(component, defaults, false, false)
    end

    publish :variable => :lastSaved, :type => "map"
    publish :variable => :autoyast_tool, :type => "boolean"
    publish :variable => :LDAPSettings, :type => "map"
    publish :variable => :LOCAL_HOST, :type => "string"
    publish :variable => :local_server, :type => "boolean"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :SetLDAPEdirSettingsinLastSaved, :type => "boolean (map)"
    publish :function => :ReadLDAPSettingsForServer, :type => "map (string)"
    publish :function => :SaveLDAPSettingsForServer, :type => "boolean (string, map)"
    publish :function => :ValidLDAPSettings, :type => "boolean (map)"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
    publish :function => :LDAP_Dialog_Opt, :type => "any (string, map, boolean, boolean)"
    publish :function => :LDAP_Dialog, :type => "any (string, map)"
  end

  OesLdap = OesLdapClass.new
  OesLdap.main
end
